package game.biome;

import game.block.BlockDirt;
import game.block.BlockDoublePlant;
import game.block.BlockTallGrass;
import game.entity.animal.EntityWolf;
import game.init.Blocks;
import game.rng.Random;
import game.world.BlockPos;
import game.world.WorldServer;
import game.worldgen.ChunkPrimer;
import game.worldgen.FeatureGenerator;
import game.worldgen.feature.WorldGenBlockBlob;
import game.worldgen.foliage.WorldGenTallGrass;
import game.worldgen.tree.WorldGenPine;
import game.worldgen.tree.WorldGenTaiga1;
import game.worldgen.tree.WorldGenTaiga2;
import game.worldgen.tree.WorldGenTree;

public class BiomeTaiga extends Biome
{
    private static final WorldGenTaiga1 field_150639_aC = new WorldGenTaiga1();
    private static final WorldGenTaiga2 field_150640_aD = new WorldGenTaiga2(false);
    private static final WorldGenPine field_150641_aE = new WorldGenPine(false, false);
    private static final WorldGenPine field_150642_aF = new WorldGenPine(false, true);
    private static final WorldGenBlockBlob field_150643_aG = new WorldGenBlockBlob(Blocks.mossy_cobblestone, 0);
    private int field_150644_aH;

    public BiomeTaiga(int id, int p_i45385_2_)
    {
        super(id);
        this.field_150644_aH = p_i45385_2_;
        this.mobs.add(new Biome.RngSpawn(EntityWolf.class, 8, 4, 4));
        this.treesPerChunk = 10;

        if (p_i45385_2_ != 1 && p_i45385_2_ != 2)
        {
            this.grassPerChunk = 1;
            this.mushroomsPerChunk = 1;
        }
        else
        {
            this.grassPerChunk = 7;
            this.deadBushPerChunk = 1;
            this.mushroomsPerChunk = 3;
        }
    }

    public WorldGenTree genBigTreeChance(Random rand)
    {
        return (WorldGenTree)((this.field_150644_aH == 1 || this.field_150644_aH == 2) && rand.chance(3) ? (this.field_150644_aH != 2 && rand.rarity(13) ? field_150641_aE : field_150642_aF) : (rand.chance(3) ? field_150639_aC : field_150640_aD));
    }

    /**
     * Gets a WorldGen appropriate for this biome.
     */
    public FeatureGenerator getRandomWorldGenForGrass(Random rand)
    {
        return rand.rarity(5) ? new WorldGenTallGrass(BlockTallGrass.EnumType.FERN) : new WorldGenTallGrass(BlockTallGrass.EnumType.GRASS);
    }

    public void decorate(WorldServer worldIn, Random rand, BlockPos pos)
    {
        if (this.field_150644_aH == 1 || this.field_150644_aH == 2)
        {
            int i = rand.zrange(3);

            for (int j = 0; j < i; ++j)
            {
                int k = rand.chOffset();
                int l = rand.chOffset();
                BlockPos blockpos = worldIn.getHeight(pos.add(k, 0, l));
                field_150643_aG.generate(worldIn, rand, blockpos);
            }
        }

        DOUBLE_PLANT_GEN.setPlantType(BlockDoublePlant.EnumPlantType.FERN);

        for (int i1 = 0; i1 < 7; ++i1)
        {
            int j1 = rand.chOffset();
            int k1 = rand.chOffset();
            int l1 = rand.zrange(worldIn.getHeight(pos.add(j1, 0, k1)).getY() + 32);
            DOUBLE_PLANT_GEN.generate(worldIn, rand, pos.add(j1, l1, k1));
        }

        super.decorate(worldIn, rand, pos);
    }

    public void genTerrainBlocks(WorldServer worldIn, Random rand, ChunkPrimer chunkPrimerIn, int x, int z, double noiseVal)
    {
        if (this.field_150644_aH == 1 || this.field_150644_aH == 2)
        {
            this.topBlock = Blocks.grass.getState();
            this.fillerBlock = Blocks.dirt.getState();

            if (noiseVal > 1.75D)
            {
                this.topBlock = Blocks.dirt.getState().withProperty(BlockDirt.VARIANT, BlockDirt.DirtType.COARSE_DIRT);
            }
            else if (noiseVal > -0.95D)
            {
                this.topBlock = Blocks.dirt.getState().withProperty(BlockDirt.VARIANT, BlockDirt.DirtType.PODZOL);
            }
        }

        this.generateBiomeTerrain(worldIn, rand, chunkPrimerIn, x, z, noiseVal);
    }

    protected Biome createMutatedBiome(int p_180277_1_)
    {
        return this.id == Biome.megaTaiga.id ? (new BiomeTaiga(p_180277_1_, 2)).setColor(5858897).setBiomeName("megaSpruceTaiga", "Hohe Fichtentaiga").setTemperatureLegacy(0.25F).setHumidity(0.8F).setHeight(new Biome.Height(this.minHeight, this.maxHeight)) : super.createMutatedBiome(p_180277_1_);
    }
}
