package game.block;

import game.init.Items;
import game.item.CheatTab;
import game.item.Item;
import game.material.Material;
import game.renderer.blockmodel.ModelBlock;
import game.rng.Random;
import game.world.State;

public class BlockMelon extends Block
{
	public BlockMelon()
    {
        super(Material.gourd);
        this.setTab(CheatTab.tabPlants);
    }

    /**
     * Get the Item that this Block should drop when harvested.
     */
    public Item getItemDropped(State state, Random rand, int fortune)
    {
        return Items.melon;
    }

    /**
     * Returns the quantity of items to drop on block destruction.
     */
    public int quantityDropped(Random random)
    {
        return random.range(3, 7);
    }

    /**
     * Get the quantity dropped based on the given fortune level
     */
    public int quantityDroppedWithBonus(int fortune, Random random)
    {
        return Math.min(9, this.quantityDropped(random) + random.zrange(1 + fortune));
    }
    
    public ModelBlock getModel(String name, State state) {
    	return new ModelBlock("melon_side").add().nswe().du("melon_top");
    }
}
