package game.block;

import game.init.Blocks;
import game.init.Config;
import game.renderer.blockmodel.ModelBlock;
import game.rng.Random;
import game.world.BlockPos;
import game.world.State;
import game.world.World;
import game.world.WorldServer;
import game.worldgen.FeatureGenerator;
import game.worldgen.foliage.WorldGenBigMushroom;

public class BlockMushroom extends BlockBush implements IGrowable
{
	public BlockMushroom()
    {
        float f = 0.2F;
        this.setBlockBounds(0.5F - f, 0.0F, 0.5F - f, 0.5F + f, f * 2.0F, 0.5F + f);
        this.setTickRandomly();
    }

    public void updateTick(WorldServer worldIn, BlockPos pos, State state, Random rand)
    {
        if (Config.shroomGrowth > 0 && rand.chance(Config.shroomGrowth))
        {
            int i = 5;
            int j = 4;

            for (BlockPos blockpos : BlockPos.getAllInBoxMutable(pos.add(-4, -1, -4), pos.add(4, 1, 4)))
            {
                if (worldIn.getState(blockpos).getBlock() == this)
                {
                    --i;

                    if (i <= 0)
                    {
                        return;
                    }
                }
            }

            BlockPos blockpos1 = pos.add(rand.zrange(3) - 1, rand.zrange(2) - rand.zrange(2), rand.zrange(3) - 1);

            for (int k = 0; k < 4; ++k)
            {
                if (worldIn.isAirBlock(blockpos1) && this.canBlockStay(worldIn, blockpos1, this.getState()))
                {
                    pos = blockpos1;
                }

                blockpos1 = pos.add(rand.zrange(3) - 1, rand.zrange(2) - rand.zrange(2), rand.zrange(3) - 1);
            }

            if (worldIn.isAirBlock(blockpos1) && this.canBlockStay(worldIn, blockpos1, this.getState()))
            {
                worldIn.setState(blockpos1, this.getState(), 2);
            }
        }
    }

    public boolean canPlaceBlockAt(World worldIn, BlockPos pos)
    {
        return super.canPlaceBlockAt(worldIn, pos) && this.canBlockStay(worldIn, pos, this.getState());
    }

    /**
     * is the block grass, dirt or farmland
     */
    protected boolean canPlaceBlockOn(Block ground)
    {
        return ground.isFullBlock();
    }

    public boolean canBlockStay(World worldIn, BlockPos pos, State state)
    {
        if (pos.getY() >= 0 && pos.getY() < 512)
        {
            State iblockstate = worldIn.getState(pos.down());
            return iblockstate.getBlock() == Blocks.mycelium ? true : (iblockstate.getBlock() == Blocks.dirt && iblockstate.getValue(BlockDirt.VARIANT) == BlockDirt.DirtType.PODZOL ? true : worldIn.getLight(pos) < 13 && this.canPlaceBlockOn(iblockstate.getBlock()));
        }
        else
        {
            return false;
        }
    }

    public boolean generateBigMushroom(WorldServer worldIn, BlockPos pos, State state, Random rand)
    {
        worldIn.setBlockToAir(pos);
        FeatureGenerator worldgenerator = null;

        if (this == Blocks.brown_mushroom)
        {
            worldgenerator = new WorldGenBigMushroom(Blocks.brown_mushroom_block);
        }
        else if (this == Blocks.red_mushroom)
        {
            worldgenerator = new WorldGenBigMushroom(Blocks.red_mushroom_block);
        }

        if (worldgenerator != null && worldgenerator.generate(worldIn, rand, pos))
        {
            return true;
        }
        else
        {
            worldIn.setState(pos, state, 3);
            return false;
        }
    }

    /**
     * Whether this IGrowable can grow
     */
    public boolean canGrow(World worldIn, BlockPos pos, State state, boolean isClient)
    {
        return true;
    }

    public boolean canUseBonemeal(World worldIn, Random rand, BlockPos pos, State state)
    {
        return (double)rand.floatv() < 0.4D;
    }

    public void grow(WorldServer worldIn, Random rand, BlockPos pos, State state)
    {
        this.generateBigMushroom(worldIn, pos, state, rand);
    }

    public ModelBlock getModel(String name, State state) {
    	return new ModelBlock(name).cross();
    }
}
