package game.block;

import game.entity.item.EntityNuke;
import game.init.Blocks;
import game.init.SoundEvent;
import game.item.CheatTab;
import game.material.Material;
import game.renderer.blockmodel.ModelBlock;
import game.world.BlockPos;
import game.world.Explosion;
import game.world.State;
import game.world.World;
import game.world.WorldServer;

public class BlockNuke extends Block
{
    public BlockNuke()
    {
        super(Material.tnt);
        this.setTab(CheatTab.tabTech);
    }

    public void onBlockAdded(WorldServer worldIn, BlockPos pos, State state)
    {
        super.onBlockAdded(worldIn, pos, state);

        if (worldIn.isBlockPowered(pos))
        {
            this.explode(worldIn, pos);
            worldIn.setBlockToAir(pos);
        }
    }
    
    public void onNeighborBlockChange(World worldIn, BlockPos pos, State state, Block neighborBlock)
    {
        if (worldIn.isBlockPowered(pos))
        {
            this.explode(worldIn, pos);
            worldIn.setBlockToAir(pos);
        }
    }
    
    public void onBlockDestroyedByExplosion(World worldIn, BlockPos pos, Explosion explosionIn, State prevState)
    {
        if (!worldIn.client)
        {
        	worldIn.setState(pos, Blocks.obsidian.getState(), 2);
        }
    }

    public void explode(World worldIn, BlockPos pos)
    {
        if (!worldIn.client)
        {
        	EntityNuke entitytntprimed = new EntityNuke(worldIn, (double)((float)pos.getX() + 0.5F), (double)pos.getY(), (double)((float)pos.getZ() + 0.5F));
            worldIn.spawnEntityInWorld(entitytntprimed);
            worldIn.playSoundAtEntity(entitytntprimed, SoundEvent.FUSE, 1.0F);
        }
    }
    
    public boolean canDropFromExplosion(Explosion explosionIn)
    {
        return false;
    }
    
    public boolean isXrayVisible()
    {
        return true;
    }

    public ModelBlock getModel(String name, State state) {
    	return new ModelBlock("nuke_side").add().nswe().d("nuke_bottom").u("nuke_top");
    }
}
