package game.block;

import java.util.List;

import game.item.CheatTab;
import game.item.Item;
import game.item.ItemStack;
import game.material.Material;
import game.properties.IProperty;
import game.properties.IStringSerializable;
import game.properties.PropertyEnum;
import game.renderer.blockmodel.ModelBlock;
import game.world.State;

public class BlockSandStone extends Block
{
    public static final PropertyEnum<BlockSandStone.EnumType> TYPE = PropertyEnum.<BlockSandStone.EnumType>create("type", BlockSandStone.EnumType.class);

    public BlockSandStone()
    {
        super(Material.rock);
        this.setDefaultState(this.getBaseState().withProperty(TYPE, BlockSandStone.EnumType.DEFAULT));
        this.setTab(CheatTab.tabNature);
    }

    /**
     * Gets the metadata of the item this Block can drop. This method is called when the block gets destroyed. It
     * returns the metadata of the dropped item based on the old metadata of the block.
     */
    public int damageDropped(State state)
    {
        return ((BlockSandStone.EnumType)state.getValue(TYPE)).getMetadata();
    }

    /**
     * returns a list of blocks with the same ID, but different meta (eg: wood returns 4 blocks)
     */
    public void getSubBlocks(Item itemIn, CheatTab tab, List<ItemStack> list)
    {
        for (BlockSandStone.EnumType blocksandstone$enumtype : BlockSandStone.EnumType.values())
        {
            list.add(new ItemStack(itemIn, 1, blocksandstone$enumtype.getMetadata()));
        }
    }

//    /**
//     * Get the MapColor for this Block and the given BlockState
//     */
//    public MapColor getMapColor(IBlockState state)
//    {
//        return MapColor.sandColor;
//    }

    /**
     * Convert the given metadata into a BlockState for this Block
     */
    public State getStateFromMeta(int meta)
    {
        return this.getState().withProperty(TYPE, BlockSandStone.EnumType.byMetadata(meta));
    }

    /**
     * Convert the BlockState into the correct metadata value
     */
    public int getMetaFromState(State state)
    {
        return ((BlockSandStone.EnumType)state.getValue(TYPE)).getMetadata();
    }

    protected IProperty[] getProperties()
    {
        return new IProperty[] {TYPE};
    }

    public ModelBlock getModel(String name, State state) {
    	switch(state.getValue(TYPE)) {
	    	case DEFAULT:
	    	default:
	    		return new ModelBlock("sandstone_normal").add().nswe().d("sandstone_bottom").u("sandstone_all");
	    	case CHISELED:
	    		return new ModelBlock("sandstone_carved").add().nswe().d("sandstone_bottom").u("sandstone_all");
	    	case SMOOTH:
	    		return new ModelBlock("sandstone_smooth").add().nswe().d("sandstone_bottom").u("sandstone_all");
		}
    }

    public static enum EnumType implements IStringSerializable
    {
        DEFAULT(0, "sandstone", "Sandstein"),
        CHISELED(1, "chiseled_sandstone", "Gemeißelter Sandstein"),
        SMOOTH(2, "smooth_sandstone", "Glatter Sandstein");

        private static final BlockSandStone.EnumType[] META_LOOKUP = new BlockSandStone.EnumType[values().length];
        private final int metadata;
        private final String name;
        private final String display;

        private EnumType(int meta, String name, String display)
        {
            this.metadata = meta;
            this.name = name;
            this.display = display;
        }

        public int getMetadata()
        {
            return this.metadata;
        }

        public String toString()
        {
            return this.name;
        }

        public static BlockSandStone.EnumType byMetadata(int meta)
        {
            if (meta < 0 || meta >= META_LOOKUP.length)
            {
                meta = 0;
            }

            return META_LOOKUP[meta];
        }

        public String getName()
        {
            return this.name;
        }

        public String getDisplay()
        {
            return this.display;
        }

        static {
            for (BlockSandStone.EnumType blocksandstone$enumtype : values())
            {
                META_LOOKUP[blocksandstone$enumtype.getMetadata()] = blocksandstone$enumtype;
            }
        }
    }
}
