package game.block;

import java.util.List;

import com.google.common.collect.Lists;

import game.init.Blocks;
import game.init.Config;
import game.init.WoodType;
import game.item.CheatTab;
import game.properties.IProperty;
import game.properties.PropertyInteger;
import game.renderer.blockmodel.ModelBlock;
import game.rng.Random;
import game.world.BlockPos;
import game.world.State;
import game.world.World;
import game.world.WorldServer;
import game.worldgen.FeatureGenerator;
import game.worldgen.tree.WorldGenBaseTree;
import game.worldgen.tree.WorldGenBigTree;
import game.worldgen.tree.WorldGenBirch;
import game.worldgen.tree.WorldGenDarkOak;
import game.worldgen.tree.WorldGenJungle;
import game.worldgen.tree.WorldGenPine;
import game.worldgen.tree.WorldGenSavanna;
import game.worldgen.tree.WorldGenTaiga2;

public class BlockSapling extends BlockBush implements IGrowable
{
//    public static final PropertyEnum<BlockPlanks.EnumType> TYPE = PropertyEnum.<BlockPlanks.EnumType>create("type", BlockPlanks.EnumType.class);
    public static final PropertyInteger STAGE = PropertyInteger.create("stage", 0, 1);
    public static final List<BlockSapling> SAPLINGS = Lists.newArrayList();
    
    private final WoodType type;

    public BlockSapling(WoodType type)
    {
        this.setDefaultState(this.getBaseState() /* .withProperty(TYPE, BlockPlanks.EnumType.OAK) */ .withProperty(STAGE, Integer.valueOf(0)));
        float f = 0.4F;
        this.setBlockBounds(0.5F - f, 0.0F, 0.5F - f, 0.5F + f, f * 2.0F, 0.5F + f);
        this.setTab(CheatTab.tabPlants);
        this.type = type;
        SAPLINGS.add(this);
    }

//    /**
//     * Gets the localized name of this block. Used for the statistics page.
//     */
//    public String getLocalizedName()
//    {
//        return I18n.translate(this.getUnlocalizedName() + "." + BlockPlanks.EnumType.OAK.getUnlocalizedName() + ".name");
//    }

    public void updateTick(WorldServer worldIn, BlockPos pos, State state, Random rand)
    {
        if(Config.saplingDry && worldIn.getTemperatureC(pos) >= 50.0f)
        {
        	worldIn.setState(pos, worldIn.rand.chance(25) ? Blocks.air.getState() : 
        			Blocks.tallgrass.getState().withProperty(BlockTallGrass.TYPE, BlockTallGrass.EnumType.DEAD_BUSH));
        	return;
        }
//        if (!worldIn.client)
//        {
        super.updateTick(worldIn, pos, state, rand);

        if (Config.treeGrowth > 0 && worldIn.getLightFromNeighbors(pos.up()) >= 9 && rand.chance(Config.treeGrowth))
        {
            this.grow(worldIn, pos, state, rand);
        }
//        }
    }

    public void grow(WorldServer worldIn, BlockPos pos, State state, Random rand)
    {
        if (((Integer)state.getValue(STAGE)).intValue() == 0)
        {
            worldIn.setState(pos, state.cycleProperty(STAGE), 4);
        }
        else
        {
            this.generateTree(worldIn, pos, state, rand);
        }
    }

    public void generateTree(WorldServer worldIn, BlockPos pos, State state, Random rand)
    {
    	WoodType type = state.getBlock() instanceof BlockSapling ? ((BlockSapling)state.getBlock()).type : WoodType.OAK;
    	State log = type == WoodType.CHERRY ? Blocks.cherry_log.getState() : // .withProperty(BlockNewLog.VARIANT, BlockPlanks.EnumType.CHERRY) :
    		(type == WoodType.MAPLE ? Blocks.maple_log.getState() /* .withProperty(BlockNewLog.VARIANT, BlockPlanks.EnumType.MAPLE) */ : Blocks.oak_log.getState());
    	State leaves = type == WoodType.CHERRY ? Blocks.cherry_leaves.getState() : 
    		(type == WoodType.MAPLE ? Blocks.maple_leaves.getState() : Blocks.oak_leaves.getState());
        FeatureGenerator worldgenerator = (FeatureGenerator)(rand.chance(10) ? new WorldGenBigTree(true, log, leaves) : new WorldGenBaseTree(true, log, leaves));
        int i = 0;
        int j = 0;
        boolean flag = false;
//        leaves = leaves.withProperty(BlockLeaves.TYPE, worldIn.getLeavesGen());

        switch (type)
        {
            case SPRUCE:
                label114:
                for (i = 0; i >= -1; --i)
                {
                    for (j = 0; j >= -1; --j)
                    {
                        if (this.isSameSaplingTypeIn(worldIn, pos, i, j, WoodType.SPRUCE))
                        {
                            worldgenerator = new WorldGenPine(false, rand.chance());
                            flag = true;
                            break label114;
                        }
                    }
                }

                if (!flag)
                {
                    j = 0;
                    i = 0;
                    worldgenerator = new WorldGenTaiga2(true);
                }

                break;

            case BIRCH:
                worldgenerator = new WorldGenBirch(true, false);
                break;

            case TIAN:
                worldgenerator = new WorldGenBigTree(true, Blocks.tian_log.getState(), Blocks.tian_leaves.getState())
                	.setHeightLimit(6, 20);
                break;

            case JUNGLE:
                State iblockstate = Blocks.jungle_log.getState(); // .withProperty(BlockOldLog.VARIANT, BlockPlanks.EnumType.JUNGLE);
                State iblockstate1 = Blocks.jungle_leaves.getState(); // .withProperty(BlockOldLeaf.VARIANT, BlockPlanks.EnumType.JUNGLE); // .withProperty(BlockLeaves.CHECK_DECAY, Boolean.valueOf(false));
                label269:

                for (i = 0; i >= -1; --i)
                {
                    for (j = 0; j >= -1; --j)
                    {
                        if (this.isSameSaplingTypeIn(worldIn, pos, i, j, WoodType.JUNGLE))
                        {
                            worldgenerator = new WorldGenJungle(true, 10, 20, iblockstate, iblockstate1);
                            flag = true;
                            break label269;
                        }
                    }
                }

                if (!flag)
                {
                    j = 0;
                    i = 0;
                    worldgenerator = new WorldGenBaseTree(true, rand.range(4, 10), iblockstate, iblockstate1, false);
                }

                break;

            case ACACIA:
                worldgenerator = new WorldGenSavanna(true);
                break;

            case DARK_OAK:
                label390:
                for (i = 0; i >= -1; --i)
                {
                    for (j = 0; j >= -1; --j)
                    {
                        if (this.isSameSaplingTypeIn(worldIn, pos, i, j, WoodType.DARK_OAK))
                        {
                            worldgenerator = new WorldGenDarkOak(true);
                            flag = true;
                            break label390;
                        }
                    }
                }

                if (!flag)
                {
                    return;
                }

            case OAK:
            case CHERRY:
            case MAPLE:
        }

        State iblockstate2 = Blocks.air.getState();

        if (flag)
        {
            worldIn.setState(pos.add(i, 0, j), iblockstate2, 4);
            worldIn.setState(pos.add(i + 1, 0, j), iblockstate2, 4);
            worldIn.setState(pos.add(i, 0, j + 1), iblockstate2, 4);
            worldIn.setState(pos.add(i + 1, 0, j + 1), iblockstate2, 4);
        }
        else
        {
            worldIn.setState(pos, iblockstate2, 4);
        }

        if (!worldgenerator.generate(worldIn, rand, pos.add(i, 0, j)))
        {
            if (flag)
            {
                worldIn.setState(pos.add(i, 0, j), state, 4);
                worldIn.setState(pos.add(i + 1, 0, j), state, 4);
                worldIn.setState(pos.add(i, 0, j + 1), state, 4);
                worldIn.setState(pos.add(i + 1, 0, j + 1), state, 4);
            }
            else
            {
                worldIn.setState(pos, state, 4);
            }
        }
    }

    private boolean isSameSaplingTypeIn(World worldIn, BlockPos pos, int xOff, int yOff, WoodType type)
    {
        return this.isTypeAt(worldIn, pos.add(xOff, 0, yOff), type) && this.isTypeAt(worldIn, pos.add(xOff + 1, 0, yOff), type) && this.isTypeAt(worldIn, pos.add(xOff, 0, yOff + 1), type) && this.isTypeAt(worldIn, pos.add(xOff + 1, 0, yOff + 1), type);
    }

    /**
     * Check whether the given BlockPos has a Sapling of the given type
     */
    public boolean isTypeAt(World worldIn, BlockPos pos, WoodType type)
    {
        State iblockstate = worldIn.getState(pos);
        return iblockstate.getBlock() instanceof BlockSapling && ((BlockSapling)iblockstate.getBlock()).type == type;
    }

//    /**
//     * Gets the metadata of the item this Block can drop. This method is called when the block gets destroyed. It
//     * returns the metadata of the dropped item based on the old metadata of the block.
//     */
//    public int damageDropped(IBlockState state)
//    {
//        return ((BlockPlanks.EnumType)state.getValue(TYPE)).getMetadata();
//    }

//    /**
//     * returns a list of blocks with the same ID, but different meta (eg: wood returns 4 blocks)
//     */
//    public void getSubBlocks(Item itemIn, CreativeTab tab, List<ItemStack> list)
//    {
//        for (BlockPlanks.EnumType blockplanks$enumtype : BlockPlanks.EnumType.values())
//        {
//            list.add(new ItemStack(itemIn, 1, blockplanks$enumtype.getMetadata()));
//        }
//    }

    /**
     * Whether this IGrowable can grow
     */
    public boolean canGrow(World worldIn, BlockPos pos, State state, boolean isClient)
    {
        return true;
    }

    public boolean canUseBonemeal(World worldIn, Random rand, BlockPos pos, State state)
    {
        return (double)worldIn.rand.floatv() < 0.45D;
    }

    public void grow(WorldServer worldIn, Random rand, BlockPos pos, State state)
    {
        this.grow(worldIn, pos, state, rand);
    }

    /**
     * Convert the given metadata into a BlockState for this Block
     */
    public State getStateFromMeta(int meta)
    {
        return this.getState() /* .withProperty(TYPE, BlockPlanks.EnumType.byMetadata(meta & 7)) */ .withProperty(STAGE, meta & 1);
    }

    /**
     * Convert the BlockState into the correct metadata value
     */
    public int getMetaFromState(State state)
    {
//        int i = 0;
//        i = i | ((BlockPlanks.EnumType)state.getValue(TYPE)).getMetadata();
//        i = i | ((Integer)state.getValue(STAGE)).intValue() << 3;
        return state.getValue(STAGE);
    }

    protected IProperty[] getProperties()
    {
        return new IProperty[] {STAGE};
    }

    public ModelBlock getModel(String name, State state) {
    	return new ModelBlock(name).cross();
    }
}
