package game.block;

import game.init.Blocks;
import game.item.CheatTab;
import game.item.Item;
import game.material.Material;
import game.properties.IProperty;
import game.properties.PropertyBool;
import game.renderer.blockmodel.ModelBlock;
import game.rng.Random;
import game.world.BlockPos;
import game.world.IWorldAccess;
import game.world.State;

public class BlockTianSoil extends Block
{
    public static final PropertyBool SNOWY = PropertyBool.create("snowy");

    public BlockTianSoil()
    {
        super(Material.rock);
        this.setDefaultState(this.getBaseState().withProperty(SNOWY, Boolean.valueOf(false)));
        this.setTab(CheatTab.tabNature);
    }
    
    public State getActualState(State state, IWorldAccess worldIn, BlockPos pos)
    {
        Block block = worldIn.getState(pos.up()).getBlock();
        return state.withProperty(SNOWY, Boolean.valueOf(block == Blocks.snow || block == Blocks.snow_layer));
    }
    
    public Item getItemDropped(State state, Random rand, int fortune)
    {
        return Blocks.tian.getItemDropped(Blocks.tian.getState(), rand, fortune);
    }

    public int getMetaFromState(State state)
    {
        return 0;
    }

    protected IProperty[] getProperties()
    {
        return new IProperty[] {SNOWY};
    }

    public ModelBlock getModel(String name, State state) {
    	if(state.getValue(SNOWY))
    		return new ModelBlock("tian_soil_side_snowed").add().nswe().d("tian").u("grass_top");
    	else
    		return new ModelBlock("tian_soil_side").add().nswe().d("tian").u("tian_soil_top");
    }
}
