package game.block;

import game.init.Blocks;
import game.init.Config;
import game.init.Items;
import game.item.CheatTab;
import game.item.Item;
import game.item.ItemStack;
import game.material.Material;
import game.properties.IProperty;
import game.properties.PropertyInteger;
import game.renderer.blockmodel.ModelBlock;
import game.rng.Random;
import game.world.BlockPos;
import game.world.State;
import game.world.World;
import game.world.WorldServer;

public class BlockWart extends BlockBush
{
    public static final PropertyInteger AGE = PropertyInteger.create("age", 0, 3);

    public BlockWart()
    {
        super(Material.plants);
        this.setDefaultState(this.getBaseState().withProperty(AGE, Integer.valueOf(0)));
        this.setTickRandomly();
        float f = 0.5F;
        this.setBlockBounds(0.5F - f, 0.0F, 0.5F - f, 0.5F + f, 0.25F, 0.5F + f);
        this.setTab((CheatTab)null);
    }

    /**
     * is the block grass, dirt or farmland
     */
    protected boolean canPlaceBlockOn(Block ground)
    {
        return ground == Blocks.soul_sand;
    }

    public boolean canBlockStay(World worldIn, BlockPos pos, State state)
    {
        return this.canPlaceBlockOn(worldIn.getState(pos.down()).getBlock());
    }

    public void updateTick(WorldServer worldIn, BlockPos pos, State state, Random rand)
    {
    	if(Config.wartGrowth > 0) {
	        int i = ((Integer)state.getValue(AGE)).intValue();
	
	        if (i < 3 && rand.chance(Config.wartGrowth))
	        {
	            state = state.withProperty(AGE, Integer.valueOf(i + 1));
	            worldIn.setState(pos, state, 2);
	        }
    	}

        super.updateTick(worldIn, pos, state, rand);
    }

    /**
     * Spawns this Block's drops into the World as EntityItems.
     */
    public void dropBlockAsItemWithChance(World worldIn, BlockPos pos, State state, float chance, int fortune)
    {
        if (!worldIn.client)
        {
            int i = 1;

            if (((Integer)state.getValue(AGE)).intValue() >= 3)
            {
                i = worldIn.rand.range(2, 4);

                if (fortune > 0)
                {
                    i += worldIn.rand.zrange(fortune + 1);
                }
            }

            for (int j = 0; j < i; ++j)
            {
                spawnAsEntity(worldIn, pos, new ItemStack(Items.soul_wart));
            }
        }
    }

    /**
     * Get the Item that this Block should drop when harvested.
     */
    public Item getItemDropped(State state, Random rand, int fortune)
    {
        return null;
    }

    /**
     * Returns the quantity of items to drop on block destruction.
     */
    public int quantityDropped(Random random)
    {
        return 0;
    }

    public Item getItem(World worldIn, BlockPos pos)
    {
        return Items.soul_wart;
    }

    /**
     * Convert the given metadata into a BlockState for this Block
     */
    public State getStateFromMeta(int meta)
    {
        return this.getState().withProperty(AGE, Integer.valueOf(meta > 3 ? 0 : meta));
    }

    /**
     * Convert the BlockState into the correct metadata value
     */
    public int getMetaFromState(State state)
    {
        return ((Integer)state.getValue(AGE)).intValue();
    }

    protected IProperty[] getProperties()
    {
        return new IProperty[] {AGE};
    }
    
    public ModelBlock getModel(String name, State state) {
    	int age = state.getValue(AGE);
    	return BlockCrops.crop(name + "_" + (age >= 2 ? age - 1 : age));
    }
}
