package game.command.commands;

import java.util.Set;

import com.google.common.collect.Sets;

import game.command.DoubleParser;
import game.command.DoubleParser.DefType;
import game.command.EnumParser;
import game.command.IntParser;
import game.command.ScriptArgs;
import game.command.ScriptEnvironment;
import game.command.ScriptException;
import game.command.ScriptExecutable;
import game.command.WorldParser;
import game.entity.Entity;
import game.entity.types.EntityLiving;
import game.init.EntityRegistry;
import game.world.Vec3;
import game.world.World;
import game.world.WorldServer;

public class CommandSpawn extends ScriptExecutable {
	public CommandSpawn() {
		super("spawn");
		Set<String> names = Sets.newTreeSet();
		for(Class<? extends Entity> clazz : EntityRegistry.getAllClasses()) {
			names.add(EntityRegistry.getEntityString(clazz));
		}
		this.addParameter("type", new EnumParser("type", null, (Object[])names.toArray(new String[names.size()])));
		this.setParamsOptional();
		this.addParameter("position", new DoubleParser("x", DefType.X, (double)(-World.MAX_SIZE), (double)World.MAX_SIZE), new DoubleParser("y", DefType.Y, 0.0, (double)World.HEIGHT), new DoubleParser("z", DefType.Z, (double)(-World.MAX_SIZE), (double)World.MAX_SIZE));
		this.addParameter("dim", new WorldParser("dim", false, true));
		
		this.addParameter("noinit", 'n');
		this.addParameter("count", 'c', new IntParser("count", false, 1, 1, 1024));
	}
	
	public Object exec(ScriptEnvironment env, ScriptArgs args) {
		String type = args.getString("type");
		WorldServer world = args.getWorld("dim");
		Vec3 pos = args.getVector("position");
		int count = args.getInt("count");
		boolean init = !args.has("noinit");
		for(int z = 0; z < count; z++) {
			Entity entity = EntityRegistry.createEntityByName(type, world);
			if(entity == null)
				throw new ScriptException("Objekt konnte nicht erzeugt werden");
			entity.setLocationAndAngles(pos.xCoord, pos.yCoord, pos.zCoord, world.rand.floatv() * 360.0f, 0.0f);
			if(init && (entity instanceof EntityLiving))
				((EntityLiving)entity).onInitialSpawn(null);
			world.spawnEntityInWorld(entity);
		}
		env.getExecutor().logConsole("%s%s bei %d, %d, %d in %s erschaffen", count == 1 ? "" : (count + "x "), EntityRegistry.getEntityName(type), (int)pos.xCoord, (int)pos.yCoord, (int)pos.zCoord, world.dimension.getFormattedName(false));
		return null;
	}
}
