package game.item;

import game.block.Block;
import game.block.BlockDirt;
import game.entity.npc.EntityNPC;
import game.init.Blocks;
import game.init.ToolMaterial;
import game.material.Material;
import game.renderer.blockmodel.Transforms;
import game.world.BlockPos;
import game.world.Facing;
import game.world.State;
import game.world.World;

public class ItemHoe extends Item
{
    protected ToolMaterial theToolMaterial;

    public ItemHoe(ToolMaterial material)
    {
        this.theToolMaterial = material;
        this.maxStackSize = 1;
        this.setMaxDamage(material.getMaxUses());
        this.setTab(CheatTab.tabTools);
    }

    

    /**
     * Called when a Block is right-clicked with this Item
     */
    public boolean onItemUse(ItemStack stack, EntityNPC playerIn, World worldIn, BlockPos pos, Facing side, float hitX, float hitY, float hitZ)
    {
        if (!playerIn.canPlayerEdit(pos.offset(side), side, stack))
        {
            return false;
        }
        else
        {
            State iblockstate = worldIn.getState(pos);
            Block block = iblockstate.getBlock();

            if (side != Facing.DOWN && worldIn.getState(pos.up()).getBlock().getMaterial() == Material.air)
            {
                if (block == Blocks.grass)
                {
                    return this.useHoe(stack, playerIn, worldIn, pos, Blocks.farmland.getState());
                }

                if (block == Blocks.dirt)
                {
                    switch ((BlockDirt.DirtType)iblockstate.getValue(BlockDirt.VARIANT))
                    {
                        case DIRT:
                            return this.useHoe(stack, playerIn, worldIn, pos, Blocks.farmland.getState());

                        case COARSE_DIRT:
                            return this.useHoe(stack, playerIn, worldIn, pos, Blocks.dirt.getState().withProperty(BlockDirt.VARIANT, BlockDirt.DirtType.DIRT));
                    }
                }
            }

            return false;
        }
    }

    protected boolean useHoe(ItemStack stack, EntityNPC player, World worldIn, BlockPos target, State newState)
    {
    	if(newState.getBlock().sound.getStepSound() != null)
        	worldIn.playSound(newState.getBlock().sound.getStepSound(), (double)((float)target.getX() + 0.5F), (double)((float)target.getY() + 0.5F), (double)((float)target.getZ() + 0.5F), 1.0F);

        if (worldIn.client)
        {
            return true;
        }
        else
        {
            worldIn.setState(target, newState);
            stack.damageItem(1, player);
            return true;
        }
    }

//    /**
//     * Returns True is the item is renderer in full 3D when hold.
//     */
//    public boolean isFull3D()
//    {
//        return true;
//    }

//    /**
//     * Returns the name of the material this tool is made from as it is declared in EnumToolMaterial (meaning diamond
//     * would return "EMERALD")
//     */
//    public String getMaterialName()
//    {
//        return this.theToolMaterial.toString();
//    }
    
    public boolean isMagnetic() {
    	return this.theToolMaterial.isMagnetic();
    }

    public Transforms getTransform() {
    	return Transforms.TOOL;
    }

    public ToolMaterial getToolMaterial()
    {
        return this.theToolMaterial;
    }
}
