package game.network;

import game.Game;
import game.audio.PositionedSound;
import game.block.Block;
import game.entity.Entity;
import game.entity.npc.EntityNPC;
import game.init.BlockRegistry;
import game.init.EntityRegistry;
import game.item.ItemBlock;
import game.item.ItemControl;
import game.item.ItemStack;
import game.material.Material;
import game.packet.CPacketAction;
import game.packet.CPacketBreak;
import game.packet.CPacketClick;
import game.packet.CPacketPlace;
import game.world.BlockPos;
import game.world.Facing;
import game.world.State;
import game.world.Vec3;
import game.world.World;
import game.world.WorldClient;

public class PlayerController
{
    private final Game gm;
    private final NetHandlerPlayClient netClientHandler;
    private BlockPos currentBlock = new BlockPos(-1, -1, -1);
    private ItemStack currentItemHittingBlock;
    private float curBlockDamageMP;
    private float stepSoundTickCounter;
    private int blockHitDelay;
    private boolean isHittingBlock;
    private boolean noclip;
    private int currentPlayerItem;
    private boolean interacting;

    public PlayerController(Game gmIn, NetHandlerPlayClient netHandler)
    {
        this.gm = gmIn;
        this.netClientHandler = netHandler;
    }

//    public static void clickBlockCreative(Game gmIn, PlayerController playerController, BlockPos pos, Facing facing)
//    {
//        if (!gmIn.theWorld.extinguishFire(gmIn.thePlayer, pos, facing))
//        {
//            playerController.onPlayerDestroyBlock(pos, facing);
//        }
//    }
    
//    public void setPlayerCapabilities()
//    {
//    	this.gm.thePlayer.flying &= this.gm.thePlayer.hasEffect(Potion.flying) || this.gm.thePlayer.noclip;
//    }
    
//    public boolean isNoclip()
//    {
//        return this.gm.thePlayer.capabilities.noClip;
//    }
    
//    public void setNoclip(boolean noclip)
//    {
//        this.noclip = noclip;
//    }
    
//    public void setCheat(boolean cheat)
//    {
//    	this.cheat = cheat;
//    	this.setPlayerCapabilities();
//    }

    

//    public boolean shouldDrawHUD()
//    {
//        return !this.creative;
//    }

    /**
     * Called when a player completes the destruction of a block
     */
    public boolean onPlayerDestroyBlock(BlockPos pos, Facing side)
    {
//        if (this.gamemode.isAdventure())
//        {
//            if (this.gamemode == Gamemode.SPECTATOR)
//            {
//                return false;
//            }

//            if (!this.gm.thePlayer.isAllowEdit())
//            {
//                Block block = this.gm.theWorld.getBlockState(pos).getBlock();
//                ItemStack itemstack = this.gm.thePlayer.getCurrentEquippedItem();
//
//                if (itemstack == null)
//                {
//                    return false;
//                }
//
//                if (!itemstack.canDestroy(block))
//                {
//                    return false;
//                }
//            }
//        }

//        if (this.gm.thePlayer.getHeldItem() != null && !this.gm.thePlayer.getHeldItem().getItem().canBreakBlocks())
//        {
//            return false;
//        }
//        else
//        {
        World world = this.gm.theWorld;
        State iblockstate = world.getState(pos);
        Block block1 = iblockstate.getBlock();

        if (block1.getMaterial() == Material.air)
        {
            return false;
        }
        else
        {
            world.playAuxSFX(2001, pos, BlockRegistry.getStateId(iblockstate));
            boolean flag = world.setBlockToAir(pos);

            if (flag)
            {
                block1.onBlockDestroyedByPlayer(world, pos, iblockstate);
            }

            this.currentBlock = new BlockPos(this.currentBlock.getX(), -1, this.currentBlock.getZ());

//            if (!this.creative)
//            {
            ItemStack itemstack1 = this.gm.thePlayer.getCurrentEquippedItem();

            if (itemstack1 != null)
            {
                itemstack1.onBlockDestroyed(world, block1, pos, this.gm.thePlayer);

                if (itemstack1.stackSize == 0)
                {
                    this.gm.thePlayer.destroyCurrentEquippedItem();
                }
            }
//            }

            return flag;
        }
//        }
    }

    /**
     * Called when the player is hitting a block with an item.
     */
    public boolean clickBlock(BlockPos loc, Facing face)
    {
//        if (this.gamemode.isAdventure())
//        {
//            if (this.gamemode == Gamemode.SPECTATOR)
//            {
//                return false;
//            }

//            if (!this.gm.thePlayer.isAllowEdit())
//            {
//                Block block = this.gm.theWorld.getBlockState(loc).getBlock();
//                ItemStack itemstack = this.gm.thePlayer.getCurrentEquippedItem();
//
//                if (itemstack == null)
//                {
//                    return false;
//                }
//
//                if (!itemstack.canDestroy(block))
//                {
//                    return false;
//                }
//            }
//        }

        if (!World.isValidXZ(loc))
        {
            return false;
        }
        else
        {
        	ItemStack stack = this.gm.thePlayer.getHeldItem();
        	if(stack != null && stack.getItem().onAction(stack, this.gm.thePlayer, this.gm.theWorld, ItemControl.PRIMARY, loc)) {
        		this.interacting = true;
        		this.netClientHandler.addToSendQueue(new CPacketBreak(CPacketBreak.Action.START_DESTROY_BLOCK, loc, face));
        		return true;
        	}
//            if (this.creative)
//            {
//                this.netClientHandler.addToSendQueue(new CPacketBreak(CPacketBreak.Action.START_DESTROY_BLOCK, loc, face));
//                clickBlockCreative(this.gm, this, loc, face);
//                this.blockHitDelay = 5;
//            }
//            else
            if (!this.isHittingBlock || !this.isHittingPosition(loc))
            {
                if (this.isHittingBlock)
                {
                    this.netClientHandler.addToSendQueue(new CPacketBreak(CPacketBreak.Action.ABORT_DESTROY_BLOCK, this.currentBlock, face));
                }

                this.netClientHandler.addToSendQueue(new CPacketBreak(CPacketBreak.Action.START_DESTROY_BLOCK, loc, face));
                Block block1 = this.gm.theWorld.getState(loc).getBlock();
                boolean flag = block1.getMaterial() != Material.air;

                if (flag && this.curBlockDamageMP == 0.0F)
                {
                    block1.onBlockClicked(this.gm.theWorld, loc, this.gm.thePlayer);
                }

                if (flag && block1.getPlayerRelativeBlockHardness(this.gm.thePlayer, this.gm.thePlayer.worldObj, loc) >= 1.0F)
                {
                    this.onPlayerDestroyBlock(loc, face);
//                	if(this.cheat && block1.getPlayerRelativeBlockHardness(this.gm.thePlayer, this.gm.thePlayer.worldObj, loc) < 1.0F)
                        this.blockHitDelay = 3;
                }
                else
                {
                    this.isHittingBlock = true;
                    this.currentBlock = loc;
                    this.currentItemHittingBlock = this.gm.thePlayer.getHeldItem();
                    this.curBlockDamageMP = 0.0F;
                    this.stepSoundTickCounter = 0.0F;
                    this.gm.theWorld.sendBlockBreakProgress(this.gm.thePlayer.getId(), this.currentBlock, (int)(this.curBlockDamageMP * 10.0F) - 1);
                }
            }

            return true;
        }
    }

    /**
     * Resets current block damage and isHittingBlock
     */
    public void resetBlockRemoving()
    {
        if (this.isHittingBlock)
        {
            this.netClientHandler.addToSendQueue(new CPacketBreak(CPacketBreak.Action.ABORT_DESTROY_BLOCK, this.currentBlock, Facing.DOWN));
            this.isHittingBlock = false;
            this.curBlockDamageMP = 0.0F;
            this.gm.theWorld.sendBlockBreakProgress(this.gm.thePlayer.getId(), this.currentBlock, -1);
        }
    }
    
    public void resetInteraction()
    {
    	this.interacting = false;
    }

    public boolean onPlayerDamageBlock(BlockPos posBlock, Facing directionFacing)
    {
    	if(this.interacting)
    		return false;
        this.syncCurrentPlayItem();

        if (this.blockHitDelay > 0)
        {
            --this.blockHitDelay;
            return true;
        }
//        else if (this.creative && World.isValidXZ(posBlock))
//        {
//            this.blockHitDelay = 5;
//            this.netClientHandler.addToSendQueue(new CPacketBreak(CPacketBreak.Action.START_DESTROY_BLOCK, posBlock, directionFacing));
//            clickBlockCreative(this.gm, this, posBlock, directionFacing);
//            return true;
//        }
        else if (this.isHittingPosition(posBlock))
        {
            Block block = this.gm.theWorld.getState(posBlock).getBlock();

            if (block.getMaterial() == Material.air)
            {
                this.isHittingBlock = false;
                return false;
            }
            else
            {
                this.curBlockDamageMP += block.getPlayerRelativeBlockHardness(this.gm.thePlayer, this.gm.thePlayer.worldObj, posBlock);

                if (this.stepSoundTickCounter % 4.0F == 0.0F && block.sound.getStepSound() != null)
                {
                    this.gm.getSoundManager().playSound(new PositionedSound(block.sound.getStepSound(), 0.25F, /* block.sound.getFrequency() * 0.5F, */ (float)posBlock.getX() + 0.5F, (float)posBlock.getY() + 0.5F, (float)posBlock.getZ() + 0.5F));
                }

                ++this.stepSoundTickCounter;

                if (this.curBlockDamageMP >= 1.0F)
                {
                    this.isHittingBlock = false;
                    this.netClientHandler.addToSendQueue(new CPacketBreak(CPacketBreak.Action.STOP_DESTROY_BLOCK, posBlock, directionFacing));
                    this.onPlayerDestroyBlock(posBlock, directionFacing);
                    this.curBlockDamageMP = 0.0F;
                    this.stepSoundTickCounter = 0.0F;
                    this.blockHitDelay = 5;
                }

                this.gm.theWorld.sendBlockBreakProgress(this.gm.thePlayer.getId(), this.currentBlock, (int)(this.curBlockDamageMP * 10.0F) - 1);
                return true;
            }
        }
        else
        {
            return this.clickBlock(posBlock, directionFacing);
        }
    }

//    /**
//     * player reach distance = 4F
//     */
//    public float getBlockReachDistance()
//    {
//        return ;
//    }

    public void updateController()
    {
        this.syncCurrentPlayItem();

        if (this.netClientHandler.getNetworkManager().isChannelOpen())
        {
            this.netClientHandler.getNetworkManager().processReceivedPackets();
        }
        else
        {
            this.netClientHandler.getNetworkManager().checkDisconnected();
        }
    }

    private boolean isHittingPosition(BlockPos pos)
    {
        ItemStack itemstack = this.gm.thePlayer.getHeldItem();
        boolean flag = this.currentItemHittingBlock == null && itemstack == null;

        if (this.currentItemHittingBlock != null && itemstack != null)
        {
            flag = itemstack.getItem() == this.currentItemHittingBlock.getItem() && ItemStack.areItemStackTagsEqual(itemstack, this.currentItemHittingBlock) && (itemstack.isItemStackDamageable() || itemstack.getMetadata() == this.currentItemHittingBlock.getMetadata());
        }

        return pos.equals(this.currentBlock) && flag;
    }

    /**
     * Syncs the current player item with the server
     */
    private void syncCurrentPlayItem()
    {
        int i = this.gm.thePlayer.inventory.currentItem;

        if (i != this.currentPlayerItem)
        {
            this.currentPlayerItem = i;
            this.netClientHandler.addToSendQueue(new CPacketAction(CPacketAction.Action.SET_ITEMSLOT, this.currentPlayerItem));
        }
    }

    public boolean onPlayerRightClick(EntityNPC player, WorldClient worldIn, ItemStack heldStack, BlockPos hitPos, Facing side, Vec3 hitVec)
    {
        this.syncCurrentPlayItem();
        float f = (float)(hitVec.xCoord - (double)hitPos.getX());
        float f1 = (float)(hitVec.yCoord - (double)hitPos.getY());
        float f2 = (float)(hitVec.zCoord - (double)hitPos.getZ());
        boolean flag = false;

        if (!World.isValidXZ(hitPos))
        {
            return false;
        }
        else
        {
        	if(heldStack == null || !heldStack.getItem().onAction(heldStack, player, worldIn, ItemControl.SECONDARY, hitPos)) {
//            if (this.gamemode != Gamemode.SPECTATOR)
//            {
                State iblockstate = worldIn.getState(hitPos);

                if ((!player.isSneaking() || player.getHeldItem() == null) // && (player.getHeldItem() == null || !player.getHeldItem().getItem().ignoresBlocks()) 
                		&& iblockstate.getBlock().onBlockActivated(worldIn, hitPos, iblockstate, player, side, f, f1, f2))
                {
                    flag = true;
                }

                if (!flag && heldStack != null && heldStack.getItem() instanceof ItemBlock)
                {
                    ItemBlock itemblock = (ItemBlock)heldStack.getItem();

                    if (!itemblock.canPlaceBlockOnSide(worldIn, hitPos, side, player, heldStack))
                    {
                        return false;
                    }
                }
//            }
        	}
        	else {
        		heldStack.getItem().onItemUse(heldStack, player, worldIn, hitPos, side, f, f1, f2);
        		flag = true;
        	}

            this.netClientHandler.addToSendQueue(new CPacketPlace(hitPos, side.getIndex(), player.inventory.getCurrentItem(), f, f1, f2));

            if (!flag) // && this.gamemode != Gamemode.SPECTATOR)
            {
                if (heldStack == null)
                {
                    return false;
                }
//                else if (this.creative)
//                {
//                    int i = heldStack.getMetadata();
//                    int j = heldStack.stackSize;
//                    boolean flag1 = heldStack.onItemUse(player, worldIn, hitPos, side, f, f1, f2);
//                    heldStack.setItemDamage(i);
//                    heldStack.stackSize = j;
//                    return flag1;
//                }
                else
                {
                    return heldStack.onItemUse(player, worldIn, hitPos, side, f, f1, f2);
                }
            }
            else
            {
                return true;
            }
        }
    }

    /**
     * Notifies the server of things like consuming food, etc...
     */
    public boolean sendUseItem(EntityNPC playerIn, World worldIn, ItemStack itemStackIn)
    {
//        if (this.gamemode == Gamemode.SPECTATOR)
//        {
//            return false;
//        }
//        else
//        {
            this.syncCurrentPlayItem();
            this.netClientHandler.addToSendQueue(new CPacketPlace(playerIn.inventory.getCurrentItem()));
            int i = itemStackIn.stackSize;
            ItemStack itemstack = itemStackIn.useItemRightClick(worldIn, playerIn);

            if (itemstack != itemStackIn || itemstack != null && itemstack.stackSize != i)
            {
                playerIn.inventory.mainInventory[playerIn.inventory.currentItem] = itemstack;

                if (itemstack.stackSize == 0)
                {
                    playerIn.inventory.mainInventory[playerIn.inventory.currentItem] = null;
                }

                return true;
            }
            else
            {
                return false;
            }
//        }
    }

    public EntityNPC createPlayerEntity(WorldClient worldIn, int type)
    {
    	EntityNPC player = (EntityNPC)EntityRegistry.createEntityByID(type, worldIn);
    	player.setClientPlayer(this.gm, this.netClientHandler);
    	return player;
    }

    /**
     * Attacks an entity
     */
    public void attackEntity(EntityNPC playerIn, Entity targetEntity)
    {
        this.syncCurrentPlayItem();
        this.netClientHandler.addToSendQueue(new CPacketAction(CPacketAction.Action.ATTACK, targetEntity.getId()));

//        if (this.gamemode != Gamemode.SPECTATOR)
//        {
        playerIn.attackTargetEntityWithCurrentItem(targetEntity);
//        }
    }

    /**
     * Send packet to server - player is interacting with another entity (left click)
     */
    public boolean interactWithEntitySendPacket(EntityNPC playerIn, Entity targetEntity)
    {
        this.syncCurrentPlayItem();
        this.netClientHandler.addToSendQueue(new CPacketAction(CPacketAction.Action.INTERACT, targetEntity.getId()));
        return /* this.gamemode != Gamemode.SPECTATOR && */ playerIn.interactWith(targetEntity);
    }

//    /**
//     * Return true when the player rightclick on an entity
//     *  
//     * @param player The player's instance
//     * @param entityIn The entity clicked
//     * @param movingObject The object clicked
//     */
//    public boolean isPlayerRightClickingOnEntity(EntityNPC player, Entity entityIn, MovingObjectPosition movingObject)
//    {
//        this.syncCurrentPlayItem();
//        Vec3 vec3 = new Vec3(movingObject.hitVec.xCoord - entityIn.posX, movingObject.hitVec.yCoord - entityIn.posY, movingObject.hitVec.zCoord - entityIn.posZ);
//        this.netClientHandler.addToSendQueue(new C02PacketUseEntity(entityIn, vec3));
//        return this.gamemode != Gamemode.SPECTATOR && entityIn.interactAt(player, vec3);
//    }

    /**
     * Handles slot clicks sends a packet to the server.
     */
    public ItemStack windowClick(int windowId, int slotId, int mouseButtonClicked, int mode, EntityNPC playerIn)
    {
        short short1 = playerIn.openContainer.getNextTransactionID(playerIn.inventory);
        ItemStack itemstack = playerIn.openContainer.slotClick(slotId, mouseButtonClicked, mode, playerIn);
        this.netClientHandler.addToSendQueue(new CPacketClick(windowId, slotId, mouseButtonClicked, mode, itemstack, short1));
        return itemstack;
    }

    /**
     * GuiEnchantment uses this during multiplayer to tell PlayerControllerMP to send a packet indicating the
     * enchantment action the player has taken.
     *  
     * @param windowID The ID of the current window
     * @param button The button id (enchantment selected)
     */
    public void sendEnchantPacket(int windowID, int button)
    {
        this.netClientHandler.addToSendQueue(new CPacketAction(CPacketAction.Action.ENCHANT_ITEM, windowID | (button << 8)));
    }

//    /**
//     * Used in PlayerControllerMP to update the server with an ItemStack in a slot.
//     */
//    public void sendCheatPacket(ItemStack itemStackIn, boolean full)
//    {
//    }

//    /**
//     * Sends a Packet107 to the server to drop the item on the ground
//     */
//    public void sendPacketDropItem(ItemStack itemStackIn)
//    {
//        if (this.creative && itemStackIn != null)
//        {
//            this.netClientHandler.addToSendQueue(new CPacketCreative(-1, itemStackIn));
//        }
//    }

    public void onStoppedUsingItem(EntityNPC playerIn)
    {
        this.syncCurrentPlayItem();
        this.netClientHandler.addToSendQueue(new CPacketBreak(CPacketBreak.Action.RELEASE_USE_ITEM, BlockPos.ORIGIN, Facing.DOWN));
        playerIn.stopUsingItem();
    }

    

//    /**
//     * Checks if the player is not creative, used for checking if it should break a block instantly
//     */
//    public boolean isNotCreative()
//    {
//        return !this.creative;
//    }

//    /**
//     * returns true if player is in creative mode
//     */
//    public boolean isCreative()
//    {
//        return this.creative;
//    }

//    /**
//     * Checks if the player is riding a horse, used to chose the GUI to open
//     */
//    public boolean isRidingHorse()
//    {
//        return ;
//    }

//    public Gamemode getGamemode()
//    {
//        return this.gamemode;
//    }

    /**
     * Return isHittingBlock
     */
    public boolean getIsHittingBlock()
    {
        return this.isHittingBlock;
    }
}
