package game.packet;

import java.io.IOException;

import game.network.NetHandlerLoginServer;
import game.network.NetHandlerPlayServer;
import game.network.Packet;
import game.network.PacketBuffer;

public class LPacketPasswordResponse implements Packet<NetHandlerLoginServer>
{
    private String user;
    private String access;
    private String password;

    public LPacketPasswordResponse()
    {
    }

    public LPacketPasswordResponse(String userIn, String accessIn, String passwordIn)
    {
        this.user = userIn;
        this.access = accessIn;
        this.password = passwordIn;
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.user = buf.readStringFromBuffer(NetHandlerPlayServer.MAX_USER_LENGTH);
        this.access = buf.readStringFromBuffer(NetHandlerPlayServer.MAX_PASS_LENGTH);
        this.password = buf.readStringFromBuffer(NetHandlerPlayServer.MAX_PASS_LENGTH);
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeString(this.user);
        buf.writeString(this.access);
        buf.writeString(this.password);
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(NetHandlerLoginServer handler)
    {
        handler.processPasswordResponse(this);
    }

    public String getUser()
    {
        return this.user;
    }

    public String getAccess()
    {
        return this.access;
    }

    public String getPassword()
    {
        return this.password;
    }
}
