package game.packet;

import java.io.IOException;

import game.network.NetHandlerPlayClient;
import game.network.Packet;
import game.network.PacketBuffer;
import game.potion.PotionEffect;

public class S1EPacketRemoveEntityEffect implements Packet<NetHandlerPlayClient>
{
    private int entityId;
    private int effectId;

    public S1EPacketRemoveEntityEffect()
    {
    }

    public S1EPacketRemoveEntityEffect(int entityIdIn, PotionEffect effect)
    {
        this.entityId = entityIdIn;
        this.effectId = effect.getPotionID();
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.entityId = buf.readVarIntFromBuffer();
        this.effectId = buf.readUnsignedByte();
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeVarIntToBuffer(this.entityId);
        buf.writeByte(this.effectId);
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(NetHandlerPlayClient handler)
    {
        handler.handleRemoveEntityEffect(this);
    }

    public int getEntityId()
    {
        return this.entityId;
    }

    public int getEffectId()
    {
        return this.effectId;
    }
}
