package game.renderer.entity;

import game.entity.animal.EntityWolf;
import game.renderer.GlState;
import game.renderer.layers.LayerWolfCollar;
import game.renderer.model.ModelBase;


public class RenderWolf extends RenderLiving<EntityWolf>
{
    private static final String wolfTextures = "textures/entity/wolf.png";
    private static final String tamedWolfTextures = "textures/entity/wolf_tame.png";
    private static final String anrgyWolfTextures = "textures/entity/wolf_angry.png";

    public RenderWolf(RenderManager renderManagerIn, ModelBase modelBaseIn)
    {
        super(renderManagerIn, modelBaseIn);
        this.addLayer(new LayerWolfCollar(this));
    }

    /**
     * Defines what float the third param in setRotationAngles of ModelBase is
     */
    protected float handleRotationFloat(EntityWolf livingBase, float partialTicks)
    {
        return livingBase.getTailRotation();
    }

    /**
     * Renders the desired {@code T} type Entity.
     */
    public void doRender(EntityWolf entity, double x, double y, double z, float partialTicks)
    {
        if (entity.isWolfWet())
        {
            float f = entity.getBrightness(partialTicks) * entity.getShadingWhileWet(partialTicks);
            GlState.color(f, f, f, 1.0F);
        }

        super.doRender(entity, x, y, z, partialTicks);
    }

    /**
     * Returns the location of an entity's texture. Doesn't seem to be called unless you call Render.bindEntityTexture.
     */
    protected String getEntityTexture(EntityWolf entity)
    {
        return entity.isTamed() ? tamedWolfTextures : (entity.isAngry() ? anrgyWolfTextures : wolfTextures);
    }
}
