package game.renderer.tileentity;

import game.Game;
import game.block.Block;
import game.block.BlockPistonBase;
import game.block.BlockPistonHead;
import game.init.Blocks;
import game.material.Material;
import game.renderer.BlockRenderer;
import game.renderer.DefaultVertexFormats;
import game.renderer.GlState;
import game.renderer.ItemRenderer;
import game.renderer.RenderBuffer;
import game.renderer.Tessellator;
import game.renderer.texture.TextureMap;
import game.tileentity.TileEntityPiston;
import game.world.BlockPos;
import game.world.State;
import game.world.World;

public class TileEntityPistonRenderer extends TileEntitySpecialRenderer<TileEntityPiston>
{
    private final BlockRenderer blockRenderer = Game.getGame().getBlockRendererDispatcher();

    public void renderTileEntityAt(TileEntityPiston te, double x, double y, double z, float partialTicks, int destroyStage)
    {
        BlockPos blockpos = te.getPos();
        State iblockstate = te.getPistonState();
        Block block = iblockstate.getBlock();

        if (block.getMaterial() != Material.air && te.getProgress(partialTicks) < 1.0F)
        {
//            Tessellator tessellator = Tessellator.getInstance();
            RenderBuffer worldrenderer = Tessellator.getBuffer();
            this.bindTexture(TextureMap.locationBlocksTexture);
            ItemRenderer.disableStandardItemLighting();
            GlState.blendFunc(770, 771);
            GlState.enableBlend();
            GlState.disableCull();

//            if (Game.getGame().isAmbientOcclusionEnabled())
//            {
//                GlState.shadeModel(7425);
//            }
//            else
//            {
            GlState.shadeModel(7424);
//            }

            worldrenderer.begin(7, DefaultVertexFormats.BLOCK);
            worldrenderer.setTranslation((double)((float)x - (float)blockpos.getX() + te.getOffsetX(partialTicks)), (double)((float)y - (float)blockpos.getY() + te.getOffsetY(partialTicks)), (double)((float)z - (float)blockpos.getZ() + te.getOffsetZ(partialTicks)));
            World world = this.getWorld();

            if (block == Blocks.piston_head && te.getProgress(partialTicks) < 0.5F)
            {
                iblockstate = iblockstate.withProperty(BlockPistonHead.SHORT, Boolean.valueOf(true));
                this.blockRenderer.renderModel(world, this.blockRenderer.getModelFromBlockState(iblockstate, world, blockpos), iblockstate, blockpos, worldrenderer, true);
            }
            else if (te.shouldPistonHeadBeRendered() && !te.isExtending())
            {
                BlockPistonHead.EnumPistonType blockpistonextension$enumpistontype = block == Blocks.sticky_piston ? BlockPistonHead.EnumPistonType.STICKY : BlockPistonHead.EnumPistonType.DEFAULT;
                State iblockstate1 = Blocks.piston_head.getState().withProperty(BlockPistonHead.TYPE, blockpistonextension$enumpistontype).withProperty(BlockPistonHead.FACING, iblockstate.getValue(BlockPistonBase.FACING));
                iblockstate1 = iblockstate1.withProperty(BlockPistonHead.SHORT, Boolean.valueOf(te.getProgress(partialTicks) >= 0.5F));
                this.blockRenderer.renderModel(world, this.blockRenderer.getModelFromBlockState(iblockstate1, world, blockpos), iblockstate1, blockpos, worldrenderer, true);
                worldrenderer.setTranslation((double)((float)x - (float)blockpos.getX()), (double)((float)y - (float)blockpos.getY()), (double)((float)z - (float)blockpos.getZ()));
                iblockstate.withProperty(BlockPistonBase.EXTENDED, Boolean.valueOf(true));
                this.blockRenderer.renderModel(world, this.blockRenderer.getModelFromBlockState(iblockstate, world, blockpos), iblockstate, blockpos, worldrenderer, true);
            }
            else
            {
                this.blockRenderer.renderModel(world, this.blockRenderer.getModelFromBlockState(iblockstate, world, blockpos), iblockstate, blockpos, worldrenderer, false);
            }

            worldrenderer.setTranslation(0.0D, 0.0D, 0.0D);
            Tessellator.draw();
            ItemRenderer.enableStandardItemLighting();
        }
    }
}
