package game.worldgen;

import game.rng.PerlinGenOld;
import game.rng.Random;

public class BiomeGenPerlin {
	private final PerlinGenOld tempNoiseGen;
	private final PerlinGenOld rainNoiseGen;
	private final PerlinGenOld multNoiseGen;
	private final double[] tempMult = new double[256];
	private final double[] temperature = new double[256];
	private final double[] humidity = new double[256];

	public BiomeGenPerlin(long seed) {
		this.tempNoiseGen = new PerlinGenOld(new Random(seed * 9871L), 4);
		this.rainNoiseGen = new PerlinGenOld(new Random(seed * 39811L), 4);
		this.multNoiseGen = new PerlinGenOld(new Random(seed * 0x84a59L), 2);
	}

	public void generate(double[] factors, int xPos, int zPos) {
		xPos *= 16;
		zPos *= 16;
		this.tempNoiseGen.generate(this.temperature, xPos, zPos, 16, 16, 0.02500000037252903D, 0.02500000037252903D, 0.25D);
		this.rainNoiseGen.generate(this.humidity, xPos, zPos, 16, 16, 0.05000000074505806D, 0.05000000074505806D, 0.33333333333333331D);
		this.multNoiseGen.generate(this.tempMult, xPos, zPos, 16, 16, 0.25D, 0.25D, 0.58823529411764708D);
		int pos = 0;
		for(int x = 0; x < 16; x++) {
			for(int z = 0; z < 16; z++) {
				double tempMul = this.tempMult[pos] * 1.1000000000000001D + 0.5D;
				double factor = 0.01D;
				double ifactor = 1.0D - factor;
				double temp = (this.temperature[pos] * 0.14999999999999999D + 0.69999999999999996D) * ifactor + tempMul * factor;
				factor = 0.002D;
				ifactor = 1.0D - factor;
				double rain = (this.humidity[pos] * 0.14999999999999999D + 0.5D) * ifactor + tempMul * factor;
				temp = 1.0D - (1.0D - temp) * (1.0D - temp);
				if(temp < 0.0D)
					temp = 0.0D;
				if(rain < 0.0D)
					rain = 0.0D;
				if(temp > 1.0D)
					temp = 1.0D;
				if(rain > 1.0D)
					rain = 1.0D;
                factors[z * 16 + x] = temp * rain;
				pos++;
			}
		}
	}
}
