package game.worldgen;

import game.biome.Biome;
import game.block.Block;
import game.init.Blocks;
import game.material.Material;
import game.rng.Random;
import game.world.BlockPos;
import game.world.LightType;
import game.world.State;
import game.world.WorldServer;

public class FeatureLakes
{
	public final State state;
	private final Block block;
	public final State filler;
//	public final Block replace;
	public final State top;
	public final int chance;
	public final int minHeight;
	public final int maxHeight;
	public final boolean ratiod;

    public FeatureLakes(State state, State filler, /* Block replace, */ State top, int chance, int minHeight, int maxHeight, 
    		boolean ratiod)
    {
        this.state = state;
        this.block = state.getBlock();
        this.filler = filler;
//        this.replace = replace;
        this.top = top;
        this.chance = chance;
        this.ratiod = ratiod;
        if(minHeight > maxHeight) {
        	this.minHeight = maxHeight;
        	this.maxHeight = minHeight;
        }
        else {
        	this.minHeight = minHeight;
        	this.maxHeight = maxHeight;
        }
    }

//    public WorldGenLakes(Block block)
//    {
//        this(block, block == Blocks.lava ? Blocks.stone.getDefaultState() : null, Blocks.dirt, Blocks.grass.getDefaultState());
//    }
    
    public boolean generate(WorldServer worldIn, Random rand, BlockPos position) {
    	if(this.ratiod) {
    		if(this.chance >= 1 && rand.chance(this.chance)) {
                int y = rand.range(this.minHeight, this.maxHeight); // rand.zrange(rand.zrange(248) + 8);
                if(y < worldIn.getSeaLevel() || rand.chance(this.chance * 5 / 4))
                	return this.generateLake(worldIn, rand, position.add(rand.chOffset(), y, rand.chOffset()));
            }
    	}
    	else {
    		if(this.chance >= 1 && rand.chance(this.chance))
    			return this.generateLake(worldIn, rand, position.add(rand.chOffset(), rand.range(this.minHeight, this.maxHeight), 
                		rand.chOffset()));
    	}
    	return false;
    }

    private boolean generateLake(WorldServer worldIn, Random rand, BlockPos position)
    {
        for (position = position.add(-8, 0, -8); position.getY() > 5 && worldIn.isAirBlock(position); position = position.down())
        {
            ;
        }

        if (position.getY() <= 4)
        {
            return false;
        }
        else
        {
            position = position.down(4);
            boolean[] aboolean = new boolean[2048];
            int i = rand.zrange(4) + 4;

            for (int j = 0; j < i; ++j)
            {
                double d0 = rand.doublev() * 6.0D + 3.0D;
                double d1 = rand.doublev() * 4.0D + 2.0D;
                double d2 = rand.doublev() * 6.0D + 3.0D;
                double d3 = rand.doublev() * (16.0D - d0 - 2.0D) + 1.0D + d0 / 2.0D;
                double d4 = rand.doublev() * (8.0D - d1 - 4.0D) + 2.0D + d1 / 2.0D;
                double d5 = rand.doublev() * (16.0D - d2 - 2.0D) + 1.0D + d2 / 2.0D;

                for (int l = 1; l < 15; ++l)
                {
                    for (int i1 = 1; i1 < 15; ++i1)
                    {
                        for (int j1 = 1; j1 < 7; ++j1)
                        {
                            double d6 = ((double)l - d3) / (d0 / 2.0D);
                            double d7 = ((double)j1 - d4) / (d1 / 2.0D);
                            double d8 = ((double)i1 - d5) / (d2 / 2.0D);
                            double d9 = d6 * d6 + d7 * d7 + d8 * d8;

                            if (d9 < 1.0D)
                            {
                                aboolean[(l * 16 + i1) * 8 + j1] = true;
                            }
                        }
                    }
                }
            }

            for (int k1 = 0; k1 < 16; ++k1)
            {
                for (int l2 = 0; l2 < 16; ++l2)
                {
                    for (int k = 0; k < 8; ++k)
                    {
                        boolean flag = !aboolean[(k1 * 16 + l2) * 8 + k] && (k1 < 15 && aboolean[((k1 + 1) * 16 + l2) * 8 + k] || k1 > 0 && aboolean[((k1 - 1) * 16 + l2) * 8 + k] || l2 < 15 && aboolean[(k1 * 16 + l2 + 1) * 8 + k] || l2 > 0 && aboolean[(k1 * 16 + (l2 - 1)) * 8 + k] || k < 7 && aboolean[(k1 * 16 + l2) * 8 + k + 1] || k > 0 && aboolean[(k1 * 16 + l2) * 8 + (k - 1)]);

                        if (flag)
                        {
                            Material material = worldIn.getState(position.add(k1, k, l2)).getBlock().getMaterial();

                            if (k >= 4 && material.isLiquid())
                            {
                                return false;
                            }

                            if (k < 4 && !material.isSolid() && worldIn.getState(position.add(k1, k, l2)).getBlock() != this.block)
                            {
                                return false;
                            }
                        }
                    }
                }
            }

            for (int l1 = 0; l1 < 16; ++l1)
            {
                for (int i3 = 0; i3 < 16; ++i3)
                {
                    for (int i4 = 0; i4 < 8; ++i4)
                    {
                        if (aboolean[(l1 * 16 + i3) * 8 + i4])
                        {
                            worldIn.setState(position.add(l1, i4, i3), i4 >= 4 ? Blocks.air.getState() : this.state, 2);
                        }
                    }
                }
            }

            if(this.top != null) {
            	Block replace = worldIn.dimension.getTop().getBlock();
	            for (int i2 = 0; i2 < 16; ++i2)
	            {
	                for (int j3 = 0; j3 < 16; ++j3)
	                {
	                    for (int j4 = 4; j4 < 8; ++j4)
	                    {
	                        if (aboolean[(i2 * 16 + j3) * 8 + j4])
	                        {
	                            BlockPos blockpos = position.add(i2, j4 - 1, j3);
	
	                            if (worldIn.getState(blockpos).getBlock() == replace && worldIn.getLightFor(LightType.SKY, position.add(i2, j4, j3)) > 0)
	                            {
	                                Biome biomegenbase = worldIn.getBiomeGenForCoords(blockpos);
	
	                                if (biomegenbase.topBlock.getBlock() == Blocks.mycelium)
	                                {
	                                    worldIn.setState(blockpos, Blocks.mycelium.getState(), 2);
	                                }
	                                else
	                                {
	                                    worldIn.setState(blockpos, this.top, 2);
	                                }
	                            }
	                        }
	                    }
	                }
	            }
            }

            if (this.filler != null) // && this.block.getMaterial() == Material.lava)
            {
                for (int j2 = 0; j2 < 16; ++j2)
                {
                    for (int k3 = 0; k3 < 16; ++k3)
                    {
                        for (int k4 = 0; k4 < 8; ++k4)
                        {
                            boolean flag1 = !aboolean[(j2 * 16 + k3) * 8 + k4] && (j2 < 15 && aboolean[((j2 + 1) * 16 + k3) * 8 + k4] || j2 > 0 && aboolean[((j2 - 1) * 16 + k3) * 8 + k4] || k3 < 15 && aboolean[(j2 * 16 + k3 + 1) * 8 + k4] || k3 > 0 && aboolean[(j2 * 16 + (k3 - 1)) * 8 + k4] || k4 < 7 && aboolean[(j2 * 16 + k3) * 8 + k4 + 1] || k4 > 0 && aboolean[(j2 * 16 + k3) * 8 + (k4 - 1)]);

                            if (flag1 && (k4 < 4 || rand.zrange(2) != 0) && worldIn.getState(position.add(j2, k4, k3)).getBlock().getMaterial().isSolid())
                            {
                                worldIn.setState(position.add(j2, k4, k3), this.filler, 2);
                            }
                        }
                    }
                }
            }

            if (this.block.getMaterial() == Material.water)
            {
                for (int k2 = 0; k2 < 16; ++k2)
                {
                    for (int l3 = 0; l3 < 16; ++l3)
                    {
                        int l4 = 3; // not 4 !!

                        if (worldIn.canBlockFreeze(position.add(k2, l4, l3), false))
                        {
                            worldIn.setState(position.add(k2, l4, l3), Blocks.ice.getState(), 2);
                        }
                    }
                }
            }

            return true;
        }
    }
}
