package game.worldgen;

import game.block.Block;
import game.material.Material;
import game.rng.Random;
import game.world.BlockPos;
import game.world.State;
import game.world.WorldServer;

public class FeatureLiquids
{
	public final State state;
	private final Block block;
//	public final Block replace;
	public final int chance;
	public final int minHeight;
	public final int maxHeight;
	public final boolean lower;

//    public WorldGenLiquids(Block block)
//    {
//        this(block, Blocks.stone);
//    }

    public FeatureLiquids(State state, /* Block replace, */ int chance, int minHeight, int maxHeight, boolean lower)
    {
        this.state = state;
        this.block = state.getBlock();
//        replace = replace;
        this.chance = chance;
        this.lower = lower;
        if(minHeight > maxHeight) {
        	this.minHeight = lower && maxHeight < 1 ? 1 : maxHeight;
        	this.maxHeight = minHeight;
        }
        else {
        	this.minHeight = lower && minHeight < 1 ? 1 : minHeight;
        	this.maxHeight = maxHeight;
        }
    }
    

    public boolean generate(WorldServer worldIn, Random rand, BlockPos position) {
    	Block replace = worldIn.dimension.getFiller().getBlock();
    	boolean flag = false;
    	for(int z = 0; z < this.chance; z++) {
    		if(this.lower)
            	// 8, 247)
                flag |= this.generateLiquid(worldIn, rand, position.add(rand.chOffset(), rand.zrange(rand.range(this.minHeight, 
                		rand.range(this.minHeight, this.maxHeight))), rand.chOffset()), replace);
    		else
//              int i17 = this.randomGenerator.range(8, 255);
	            flag |= this.generateLiquid(worldIn, rand, position.add(rand.chOffset(), rand.range(this.minHeight, 
	            		rand.range(this.minHeight, this.maxHeight)), rand.chOffset()), replace);
        }
        return flag;
    }

    private boolean generateLiquid(WorldServer worldIn, Random rand, BlockPos position, Block replace)
    {
        if (worldIn.getState(position.up()).getBlock() != replace)
        {
            return false;
        }
        else if (worldIn.getState(position.down()).getBlock() != replace)
        {
            return false;
        }
        else if (worldIn.getState(position).getBlock().getMaterial() != Material.air && worldIn.getState(position).getBlock() != replace)
        {
            return false;
        }
        else
        {
            int i = 0;

            if (worldIn.getState(position.west()).getBlock() == replace)
            {
                ++i;
            }

            if (worldIn.getState(position.east()).getBlock() == replace)
            {
                ++i;
            }

            if (worldIn.getState(position.north()).getBlock() == replace)
            {
                ++i;
            }

            if (worldIn.getState(position.south()).getBlock() == replace)
            {
                ++i;
            }

            int j = 0;

            if (worldIn.isAirBlock(position.west()))
            {
                ++j;
            }

            if (worldIn.isAirBlock(position.east()))
            {
                ++j;
            }

            if (worldIn.isAirBlock(position.north()))
            {
                ++j;
            }

            if (worldIn.isAirBlock(position.south()))
            {
                ++j;
            }

            if (i == 3 && j == 1)
            {
                worldIn.setState(position, this.state, 2);
                worldIn.forceBlockUpdateTick(this.block, position, rand);
            }

            return true;
        }
    }
}
