package game.worldgen;

import game.block.Block;
import game.rng.Random;
import game.util.ExtMath;
import game.world.BlockPos;
import game.world.State;
import game.world.WorldServer;

public class FeatureOres
{
	public final State oreBlock;
//    private final Block replace;
    public final boolean distributed;
    private final int spawnTries;
    private final int moreTries;
    public final int numberOfBlocks;
    private final int minHeight;
    private final int maxHeight;

    public final int spawns;
    public final int min;
    public final int max;
    public final int more;
    
//    public FeatureOres(IBlockState state, boolean dist, int spawns, int count, int min, int max) {
//    	this(state, dist, spawns, 0, count, min, max);
//    }

//    public FeatureOres(IBlockState state, boolean dist, int spawns, int more, int count, int min, int max) {
//    	this(state, dist, spawns, more, count, min, max, null);
//    }

    public FeatureOres(State state, int count, int more, int size, int min, int max, boolean dist) // , Block replace)
    {
    	this.spawns = count;
    	this.min = min;
    	this.max = max;
    	this.more = more;
    	count = count == 2 ? 3 : count;
        this.oreBlock = state;
        this.distributed = dist;
        if(this.distributed) {
        	min = min < 0 ? 0 : (min > 511 ? 511 : min);
        	max = max < 0 ? 0 : (max > 512 ? 512 : max);
        	if(min + max > 511) {
        		max = 512 - min;
        	}
        	if(min - max < 0) {
        		max = min;
        	}
        }
        else {
        	min = min < 0 ? 0 : (min > 512 ? 512 : min);
        	max = max < 0 ? 0 : (max > 512 ? 512 : max);
        	if (max < min)
            {
                int i = min;
                min = max;
                max = i;
            }
            else if (max == min)
            {
                if (min < 511)
                {
                    ++max;
                }
                else
                {
                    --min;
                }
            }
        }
        this.minHeight = min;
        this.maxHeight = max;
        int minTries = count <= 0 ? 1 : 0;
        this.spawnTries = count < 0 ? 0 : count;
        this.moreTries = more < minTries ? minTries : more;
        this.numberOfBlocks = size < 1 ? 1 : size;
//        this.replace = replace;
    }
    
    public boolean generate(WorldServer worldIn, Random rand, BlockPos position) {
    	Block replace = /* this.replace != null ? this.replace : */ worldIn.dimension.getFiller().getBlock();
    	int tries = this.spawnTries == 0 ? 
    			(rand.zrange(this.moreTries) == 0 ? 1 : 0) : 
    			(this.spawnTries + (this.moreTries == 0 ? 0 : rand.zrange(this.moreTries + 1)));
//    	tries = tries == 0 ? (this.numberOfBlocks == 1 ? (rand.nextInt(10) == 0 ? 1 : 0) : rand.nextInt(2)) : tries;
    	for(int j = 0; j < tries; j++) {
            BlockPos blockpos;
            if(this.distributed) {
            	blockpos = position.add(rand.zrange(16), rand.zrange(this.maxHeight) + rand.zrange(this.maxHeight) + this.minHeight - this.maxHeight, rand.zrange(16));
            }
            else {
            	blockpos = position.add(rand.zrange(16), rand.zrange(this.maxHeight - this.minHeight) + this.minHeight, rand.zrange(16));
            }
            if(this.numberOfBlocks == 1) {
            	 if(worldIn.getState(blockpos).getBlock() == replace) {
                     worldIn.setState(blockpos, this.oreBlock, 2);
                 }
            }
            else { // if(position.getX() >= -67000000 && position.getZ() >= -67000000 && position.getX() <= 67000000 && position.getZ() <= 67000000) {
            	this.generateVein(worldIn, rand, blockpos, replace); // Fix dist
            }
        }
    	return true;
    }

    private void generateVein(WorldServer worldIn, Random rand, BlockPos position, Block replace)
    {
        float f = rand.floatv() * (float)Math.PI;
        double d0 = (double)(/* (float)(position.getX() + 8) */ 8.0f + ExtMath.sin(f) * (float)this.numberOfBlocks / 8.0F);
        double d1 = (double)(/* (float)(position.getX() + 8) */ 8.0f - ExtMath.sin(f) * (float)this.numberOfBlocks / 8.0F);
        double d2 = (double)(/* (float)(position.getZ() + 8) */ 8.0f + ExtMath.cos(f) * (float)this.numberOfBlocks / 8.0F);
        double d3 = (double)(/* (float)(position.getZ() + 8) */ 8.0f - ExtMath.cos(f) * (float)this.numberOfBlocks / 8.0F);
        double d4 = (double)(/* position.getY() + */ rand.zrange(3) - 2);
        double d5 = (double)(/* position.getY() + */ rand.zrange(3) - 2);

        for (int i = 0; i < this.numberOfBlocks; ++i)
        {
            float f1 = (float)i / (float)this.numberOfBlocks;
            double d6 = d0 + (d1 - d0) * (double)f1;
            double d7 = d4 + (d5 - d4) * (double)f1;
            double d8 = d2 + (d3 - d2) * (double)f1;
            double d9 = rand.doublev() * (double)this.numberOfBlocks / 16.0D;
            double d10 = (double)(ExtMath.sin((float)Math.PI * f1) + 1.0F) * d9 + 1.0D;
            double d11 = (double)(ExtMath.sin((float)Math.PI * f1) + 1.0F) * d9 + 1.0D;
            int j = ExtMath.floord(d6 - d10 / 2.0D);
            int k = ExtMath.floord(d7 - d11 / 2.0D);
            int l = ExtMath.floord(d8 - d10 / 2.0D);
            int i1 = ExtMath.floord(d6 + d10 / 2.0D);
            int j1 = ExtMath.floord(d7 + d11 / 2.0D);
            int k1 = ExtMath.floord(d8 + d10 / 2.0D);

            for (int l1 = j; l1 <= i1; ++l1)
            {
                double d12 = ((double)l1 + 0.5D - d6) / (d10 / 2.0D);

                if (d12 * d12 < 1.0D)
                {
                    for (int i2 = k; i2 <= j1; ++i2)
                    {
                        double d13 = ((double)i2 + 0.5D - d7) / (d11 / 2.0D);

                        if (d12 * d12 + d13 * d13 < 1.0D)
                        {
                            for (int j2 = l; j2 <= k1; ++j2)
                            {
                                double d14 = ((double)j2 + 0.5D - d8) / (d10 / 2.0D);

                                if (d12 * d12 + d13 * d13 + d14 * d14 < 1.0D)
                                {
                                    BlockPos blockpos = position.add(l1, i2, j2); // new BlockPos(l1, i2, j2);

                                    if (worldIn.getState(blockpos).getBlock() == replace)
                                    {
                                        worldIn.setState(blockpos, this.oreBlock, 2);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
