package game.worldgen.feature;

import game.block.Block;
import game.init.Blocks;
import game.item.RngLoot;
import game.material.Material;
import game.rng.Random;
import game.rng.WeightedList;
import game.tileentity.TileEntity;
import game.tileentity.TileEntityChest;
import game.world.BlockPos;
import game.world.WorldServer;
import game.worldgen.FeatureGenerator;
import game.worldgen.LootConstants;

public class WorldGenAbandonedChest extends FeatureGenerator
{
    private final WeightedList<RngLoot> items;
    private final int amount;
    
    // BlockPos pos = world.getTopSolidOrLiquidBlock(new BlockPos(x, 0, z)).up();

    public WorldGenAbandonedChest(WeightedList<RngLoot> items, int amount)
    {
        this.items = items;
        this.amount = amount;
    }
    
    public WorldGenAbandonedChest() {
    	this(LootConstants.ABANDONED_ITEMS, 10);
    }

    public boolean generate(WorldServer worldIn, Random rand, BlockPos position)
    {
        Block block;
        while (((block = worldIn.getState(position).getBlock()).getMaterial() == Material.air || block.getMaterial() == Material.leaves) && position.getY() > 1)
        {
            position = position.down();
        }
        if (position.getY() < 1)
        {
            return false;
        }
        else
        {
            position = position.up();
            for (int i = 0; i < 4; ++i)
            {
                BlockPos blockpos = position.add(rand.zrange(4) - rand.zrange(4), rand.zrange(3) - rand.zrange(3), rand.zrange(4) - rand.zrange(4));
                if (worldIn.isAirBlock(blockpos) && worldIn.isBlockSolid(blockpos.down()))
                {
                    worldIn.setState(blockpos, Blocks.chest.getState(), 2);
                    TileEntity tileentity = worldIn.getTileEntity(blockpos);
                    if(tileentity instanceof TileEntityChest)
                        RngLoot.generateChestContents(rand, this.items, (TileEntityChest)tileentity, this.amount);
                    return true;
                }
            }
            return false;
        }
    }
}
