package game.worldgen.feature;

import game.rng.Random;
import game.world.BlockPos;
import game.world.State;
import game.world.WorldServer;
import game.worldgen.FeatureGenerator;

public class WorldGenAsteroid extends FeatureGenerator {
	private final State[] blocks;

	public WorldGenAsteroid(State ... blocks) {
		this.blocks = blocks;
	}

	public boolean generate(WorldServer world, Random rand, BlockPos pos) {
		for(pos = pos.add(-8, 0, -8); pos.getY() > 9 && !world.isAirBlock(pos); pos = pos.down()) {
			;
		}
		if(pos.getY() <= 8)
			return false;
		pos = pos.down(8);
		boolean[] set = new boolean[4096];
		int iter = rand.range(1, 15);
		for(int n = 0; n < iter; ++n) {
			double d0 = rand.doublev() * 6.0D + 3.0D;
			double d1 = rand.doublev() * 6.0D + 3.0D;
			double d2 = rand.doublev() * 6.0D + 3.0D;
			double d3 = rand.doublev() * (16.0D - d0 - 2.0D) + 1.0D + d0 / 2.0D;
			double d4 = rand.doublev() * (16.0D - d1 - 2.0D) + 1.0D + d1 / 2.0D;
			double d5 = rand.doublev() * (16.0D - d2 - 2.0D) + 1.0D + d2 / 2.0D;
			for(int x = 1; x < 15; ++x) {
				for(int z = 1; z < 15; ++z) {
					for(int y = 1; y < 15; ++y) {
						double dx = ((double)x - d3) / (d0 / 2.0D);
						double dy = ((double)y - d4) / (d1 / 2.0D);
						double dz = ((double)z - d5) / (d2 / 2.0D);
						double dist = dx * dx + dy * dy + dz * dz;
						if(dist < 1.0D) {
							set[(x * 16 + z) * 16 + y] = true;
						}
					}
				}
			}
		}
		for(int x = 0; x < 16; ++x) {
			for(int z = 0; z < 16; ++z) {
				for(int y = 0; y < 16; ++y) {
					boolean flag = !set[(x * 16 + z) * 16 + y]
							&& (x < 15 && set[((x + 1) * 16 + z) * 16 + y] || x > 0 && set[((x - 1) * 16 + z) * 16 + y]
									|| z < 15 && set[(x * 16 + z + 1) * 16 + y] || z > 0 && set[(x * 16 + (z - 1)) * 16 + y]
									|| y < 7 && set[(x * 16 + z) * 16 + y + 1] || y > 0 && set[(x * 16 + z) * 16 + (y - 1)]);
					if(flag && !world.isAirBlock(pos.add(x, y, z))) {
						return false;
					}
				}
			}
		}
		for(int x = 0; x < 16; ++x) {
			for(int z = 0; z < 16; ++z) {
				for(int y = 0; y < 16; ++y) {
					if(set[(x * 16 + z) * 16 + y]) {
						world.setState(pos.add(x, y, z), rand.pick(this.blocks), 2);
					}
				}
			}
		}
		return true;
	}
}
