package game.worldgen.feature;

import game.block.Block;
import game.init.Blocks;
import game.rng.Random;
import game.world.BlockPos;
import game.world.WorldServer;
import game.worldgen.FeatureGenerator;

public class WorldGenClay extends FeatureGenerator
{
    protected Block clayBlock = Blocks.clay;

    /** The number of blocks to generate. */
    protected int numberOfBlocks;

    public WorldGenClay(int blocks)
    {
        this.numberOfBlocks = blocks;
    }

    public boolean generate(WorldServer worldIn, Random rand, BlockPos position)
    {
        if (!worldIn.getState(position).getBlock().getMaterial().isColdLiquid())
        {
            return false;
        }
        else
        {
            int i = rand.zrange(this.numberOfBlocks - 2) + 2;
            int j = 1;

            for (int k = position.getX() - i; k <= position.getX() + i; ++k)
            {
                for (int l = position.getZ() - i; l <= position.getZ() + i; ++l)
                {
                    int i1 = k - position.getX();
                    int j1 = l - position.getZ();

                    if (i1 * i1 + j1 * j1 <= i * i)
                    {
                        for (int k1 = position.getY() - j; k1 <= position.getY() + j; ++k1)
                        {
                            BlockPos blockpos = new BlockPos(k, k1, l);
                            Block block = worldIn.getState(blockpos).getBlock();

                            if (block == Blocks.dirt || block == Blocks.clay)
                            {
                                worldIn.setState(blockpos, this.clayBlock.getState(), 2);
                            }
                        }
                    }
                }
            }

            return true;
        }
    }
}
