package game.worldgen.feature;

import game.block.Block;
import game.entity.Entity;
import game.entity.item.EntityCrystal;
import game.rng.Random;
import game.world.BlockPos;
import game.world.State;
import game.world.WorldServer;
import game.worldgen.FeatureGenerator;

public class WorldGenSpikes extends FeatureGenerator
{
    private final Block base;
    private final int maxHeight;
    private final int minRadius;
    private final int maxRadius;
    private final State block;
    private final boolean crystals;

    public WorldGenSpikes(Block base, int maxHeight, int minRadius, int maxRadius, State placed, boolean crystals)
    {
        this.base = base;
        this.maxHeight = maxHeight;
        this.minRadius = minRadius;
        this.maxRadius = maxRadius;
        this.block = placed;
        this.crystals = crystals;
    }

    public boolean generate(WorldServer worldIn, Random rand, BlockPos position)
    {
        if (worldIn.isAirBlock(position) && worldIn.getState(position.down()).getBlock() == this.base)
        {
            int i = rand.zrange(this.maxHeight - 6) + 6;
            int j = rand.range(this.minRadius, this.maxRadius);
            BlockPos.MutableBlockPos blockpos$mutableblockpos = new BlockPos.MutableBlockPos();

            for (int k = position.getX() - j; k <= position.getX() + j; ++k)
            {
                for (int l = position.getZ() - j; l <= position.getZ() + j; ++l)
                {
                    int i1 = k - position.getX();
                    int j1 = l - position.getZ();

                    if (i1 * i1 + j1 * j1 <= j * j + 1 && worldIn.getState(blockpos$mutableblockpos.set(k, position.getY() - 1, l)).getBlock() != this.base)
                    {
                        return false;
                    }
                }
            }

            for (int l1 = position.getY(); l1 < position.getY() + i && l1 < 512; ++l1)
            {
                for (int i2 = position.getX() - j; i2 <= position.getX() + j; ++i2)
                {
                    for (int j2 = position.getZ() - j; j2 <= position.getZ() + j; ++j2)
                    {
                        int k2 = i2 - position.getX();
                        int k1 = j2 - position.getZ();

                        if (k2 * k2 + k1 * k1 <= j * j + 1)
                        {
                            worldIn.setState(new BlockPos(i2, l1, j2), this.block, 2);
                        }
                    }
                }
            }

            if(this.crystals) {
	            Entity entity = new EntityCrystal(worldIn);
	            entity.setLocationAndAngles((double)((float)position.getX() + 0.5F), (double)(position.getY() + i), (double)((float)position.getZ() + 0.5F), rand.floatv() * 360.0F, 0.0F);
	            worldIn.spawnEntityInWorld(entity);
//	            if(worldIn.dimension.getDimensionId() == 1)
//	            	worldIn.setBlockState(position.up(i), Blocks.obsidian.getDefaultState(), 2);
            }
            return true;
        }
        else
        {
            return false;
        }
    }
}
