package game.worldgen.layer;

import game.biome.Biome;

public abstract class GenLayer {
	private long worldGenSeed;
	private long chunkSeed;
	private long baseSeed;
	protected GenLayer parent;

	protected static boolean canBeNearby(int id1, int id2) {
		if(id1 == id2) {
			return true;
		}
		else { // if(id1 != Biome.mesaPlateau_F.id && id1 != Biome.mesaPlateau.id) {
			final Biome biome1 = Biome.getBiome(id1);
			final Biome biome2 = Biome.getBiome(id2);
			return biome1 != null && biome2 != null ? biome1.isEqualTo(biome2) : false;
		}
//		else {
//			return id2 == Biome.mesaPlateau_F.id || id2 == Biome.mesaPlateau.id;
//		}
	}

	protected static boolean isSea(int id) {
		return id == Biome.sea.id || id == Biome.frozenSea.id;
	}

	public GenLayer(long base) {
		this.baseSeed = base;
		this.baseSeed *= this.baseSeed * 6364136223846793005L + 1442695040888963407L;
		this.baseSeed += base;
		this.baseSeed *= this.baseSeed * 6364136223846793005L + 1442695040888963407L;
		this.baseSeed += base;
		this.baseSeed *= this.baseSeed * 6364136223846793005L + 1442695040888963407L;
		this.baseSeed += base;
	}

	public abstract int[] getInts(int areaX, int areaY, int areaWidth, int areaHeight);

	public void initWorldGenSeed(long seed) {
		this.worldGenSeed = seed;
		if(this.parent != null) {
			this.parent.initWorldGenSeed(seed);
		}
		this.worldGenSeed *= this.worldGenSeed * 6364136223846793005L + 1442695040888963407L;
		this.worldGenSeed += this.baseSeed;
		this.worldGenSeed *= this.worldGenSeed * 6364136223846793005L + 1442695040888963407L;
		this.worldGenSeed += this.baseSeed;
		this.worldGenSeed *= this.worldGenSeed * 6364136223846793005L + 1442695040888963407L;
		this.worldGenSeed += this.baseSeed;
	}

	public void initChunkSeed(long x, long z) {
		this.chunkSeed = this.worldGenSeed;
		this.chunkSeed *= this.chunkSeed * 6364136223846793005L + 1442695040888963407L;
		this.chunkSeed += x;
		this.chunkSeed *= this.chunkSeed * 6364136223846793005L + 1442695040888963407L;
		this.chunkSeed += z;
		this.chunkSeed *= this.chunkSeed * 6364136223846793005L + 1442695040888963407L;
		this.chunkSeed += x;
		this.chunkSeed *= this.chunkSeed * 6364136223846793005L + 1442695040888963407L;
		this.chunkSeed += z;
	}

	protected int nextInt(int range) {
		int i = (int)((this.chunkSeed >> 24) % (long)range);
		if(i < 0) {
			i += range;
		}
		this.chunkSeed *= this.chunkSeed * 6364136223846793005L + 1442695040888963407L;
		this.chunkSeed += this.worldGenSeed;
		return i;
	}

	protected int getRandom(int... ids) {
		return ids[this.nextInt(ids.length)];
	}

	protected int getFrequent(int a, int b, int c, int d) {
		return 
			b == c && c == d ? b : 
			(a == b && a == c ? a : 
			(a == b && a == d ? a : 
			(a == c && a == d ? a : 
			(a == b && c != d ? a : 
			(a == c && b != d ? a : 
			(a == d && b != c ? a : 
			(b == c && a != d ? b : 
			(b == d && a != c ? b : 
			(c == d && a != b ? c : 
			this.getRandom(a, b, c, d)
		)))))))));
	}
}
