package game.worldgen.layer;

import game.biome.Biome;

public class GenLayerRiverMix extends GenLayer
{
    private GenLayer biomePatternGeneratorChain;
    private GenLayer riverPatternGeneratorChain;
    private final int def;

    public GenLayerRiverMix(long p_i2129_1_, GenLayer p_i2129_3_, GenLayer p_i2129_4_, Biome def)
    {
        super(p_i2129_1_);
        this.biomePatternGeneratorChain = p_i2129_3_;
        this.riverPatternGeneratorChain = p_i2129_4_;
        this.def = def.id;
    }

    /**
     * Initialize layer's local worldGenSeed based on its own baseSeed and the world's global seed (passed in as an
     * argument).
     */
    public void initWorldGenSeed(long seed)
    {
        this.biomePatternGeneratorChain.initWorldGenSeed(seed);
        this.riverPatternGeneratorChain.initWorldGenSeed(seed);
        super.initWorldGenSeed(seed);
    }

    /**
     * Returns a list of integer values generated by this layer. These may be interpreted as temperatures, rainfall
     * amounts, or biomeList[] indices based on the particular GenLayer subclass.
     */
    public int[] getInts(int areaX, int areaY, int areaWidth, int areaHeight)
    {
        int[] biome = this.biomePatternGeneratorChain.getInts(areaX, areaY, areaWidth, areaHeight);
        int[] river = this.riverPatternGeneratorChain.getInts(areaX, areaY, areaWidth, areaHeight);
        int[] out = IntCache.getIntCache(areaWidth * areaHeight);

        for (int i = 0; i < areaWidth * areaHeight; ++i)
        {
            if(biome[i] == Biome.none.id)
            {
            	out[i] = this.def;
            }
            else if(biome[i] == Biome.sea.id || biome[i] == Biome.frozenSea.id)
            {
            	out[i] = biome[i];
            }
            else if (river[i] == Biome.river.id)
            {
                if (biome[i] == Biome.icePlains.id)
                {
                    out[i] = Biome.frozenRiver.id;
                }
                else // if (biome[i] != Biome.mushroomPlains.id && biome[i] != Biome.mushroomPlainsEdge.id)
                {
                    out[i] = Biome.river.id;
                }
//                else
//                {
//                    out[i] = Biome.mushroomPlainsEdge.id;
//                }
            }
            else
            {
                out[i] = biome[i];
            }
        }

        return out;
    }
}
