package game.worldgen.tree;

import game.block.BlockVine;
import game.init.Blocks;
import game.properties.PropertyBool;
import game.rng.Random;
import game.util.ExtMath;
import game.world.BlockPos;
import game.world.State;
import game.world.WorldServer;

public class WorldGenJungle extends WorldGenHugeTree
{
    public WorldGenJungle(boolean p_i46448_1_, int p_i46448_2_, int p_i46448_3_, State p_i46448_4_, State p_i46448_5_)
    {
        super(p_i46448_1_, p_i46448_2_, p_i46448_3_, p_i46448_4_, p_i46448_5_);
    }

    public boolean generate(WorldServer worldIn, Random rand, BlockPos position)
    {
        int i = this.func_150533_a(rand);

        if (!this.func_175929_a(worldIn, rand, position, i))
        {
            return false;
        }
        else
        {
            this.func_175930_c(worldIn, position.up(i), 2);

            for (int j = position.getY() + i - 2 - rand.zrange(4); j > position.getY() + i / 2; j -= 2 + rand.zrange(4))
            {
                float f = rand.floatv() * (float)Math.PI * 2.0F;
                int k = position.getX() + (int)(0.5F + ExtMath.cos(f) * 4.0F);
                int l = position.getZ() + (int)(0.5F + ExtMath.sin(f) * 4.0F);

                for (int i1 = 0; i1 < 5; ++i1)
                {
                    k = position.getX() + (int)(1.5F + ExtMath.cos(f) * (float)i1);
                    l = position.getZ() + (int)(1.5F + ExtMath.sin(f) * (float)i1);
                    this.setBlockAndNotifyAdequately(worldIn, new BlockPos(k, j - 3 + i1 / 2, l), this.woodMetadata);
                }

                int j2 = 1 + rand.zrange(2);
                int j1 = j;

                for (int k1 = j - j2; k1 <= j1; ++k1)
                {
                    int l1 = k1 - j1;
                    this.func_175928_b(worldIn, new BlockPos(k, k1, l), 1 - l1);
                }
            }

            for (int i2 = 0; i2 < i; ++i2)
            {
                BlockPos blockpos = position.up(i2);

                if (this.canBeReplaced(worldIn.getState(blockpos).getBlock()))
                {
                    this.setBlockAndNotifyAdequately(worldIn, blockpos, this.woodMetadata);

                    if (i2 > 0)
                    {
                        this.func_181632_a(worldIn, rand, blockpos.west(), BlockVine.EAST);
                        this.func_181632_a(worldIn, rand, blockpos.north(), BlockVine.SOUTH);
                    }
                }

                if (i2 < i - 1)
                {
                    BlockPos blockpos1 = blockpos.east();

                    if (this.canBeReplaced(worldIn.getState(blockpos1).getBlock()))
                    {
                        this.setBlockAndNotifyAdequately(worldIn, blockpos1, this.woodMetadata);

                        if (i2 > 0)
                        {
                            this.func_181632_a(worldIn, rand, blockpos1.east(), BlockVine.WEST);
                            this.func_181632_a(worldIn, rand, blockpos1.north(), BlockVine.SOUTH);
                        }
                    }

                    BlockPos blockpos2 = blockpos.south().east();

                    if (this.canBeReplaced(worldIn.getState(blockpos2).getBlock()))
                    {
                        this.setBlockAndNotifyAdequately(worldIn, blockpos2, this.woodMetadata);

                        if (i2 > 0)
                        {
                            this.func_181632_a(worldIn, rand, blockpos2.east(), BlockVine.WEST);
                            this.func_181632_a(worldIn, rand, blockpos2.south(), BlockVine.NORTH);
                        }
                    }

                    BlockPos blockpos3 = blockpos.south();

                    if (this.canBeReplaced(worldIn.getState(blockpos3).getBlock()))
                    {
                        this.setBlockAndNotifyAdequately(worldIn, blockpos3, this.woodMetadata);

                        if (i2 > 0)
                        {
                            this.func_181632_a(worldIn, rand, blockpos3.west(), BlockVine.EAST);
                            this.func_181632_a(worldIn, rand, blockpos3.south(), BlockVine.NORTH);
                        }
                    }
                }
            }

            return true;
        }
    }

    private void func_181632_a(WorldServer p_181632_1_, Random p_181632_2_, BlockPos p_181632_3_, PropertyBool p_181632_4_)
    {
        if (p_181632_2_.zrange(3) > 0 && p_181632_1_.isAirBlock(p_181632_3_))
        {
            this.setBlockAndNotifyAdequately(p_181632_1_, p_181632_3_, Blocks.vine.getState().withProperty(p_181632_4_, Boolean.valueOf(true)));
        }
    }

    private void func_175930_c(WorldServer worldIn, BlockPos p_175930_2_, int p_175930_3_)
    {
        int i = 2;

        for (int j = -i; j <= 0; ++j)
        {
            this.func_175925_a(worldIn, p_175930_2_.up(j), p_175930_3_ + 1 - j);
        }
    }
}
